###################################################################################################
# _accessmanager.py
#
# $Id: _accessmanager.py,v 1.1 2003/06/10 12:52:31 nordmann Exp $
# $Name:  $
# $Author: nordmann $
# $Revision: 1.1 $
#
# Implementation of class AccessManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
from AccessControl.User import UserFolder
import urllib
# Product Imports.
import _globals


# -------------------------------------------------------------------------------------------
#  role_defs:
#
#  Role Definitions
# -------------------------------------------------------------------------------------------
role_defs = {
   'ZMSAdministrator':{}
  ,'ZMSEditor':{'revoke':['Change DTML Methods','ZMS Administrator']}
  ,'ZMSAuthor':{'revoke':['Change DTML Methods','Import/Export objects','ZMS Administrator','ZMS Editor']}
  ,'ZMSSubscriber':{'grant':['Access contents information','View']}
}

# -------------------------------------------------------------------------------------------
#  role_permissions:
#
#  Role Permissions
# -------------------------------------------------------------------------------------------
def role_permissions(self, role):
  permissions = map(lambda x: x['name'],self.permissionsOfRole('Manager'))
  if role_defs.has_key(role):
    role_def = role_defs[role]
    if role_def.has_key('revoke'):
      for revoke in role_def['revoke']:
        if revoke in permissions:
          permissions.remove(revoke)
    elif role_def.has_key('grant'):
      permissions = role_def['grant']
  return permissions

# -------------------------------------------------------------------------------------------
#  getRoles:
#
#  Get Roles for different user-folder types.
# -------------------------------------------------------------------------------------------
def getRoles(self, name):
  roles = []
  userFldr = self.getUserFolder()
  if userFldr is not None:
    if userFldr.meta_type == 'User Folder':
      user = userFldr.getUser(name)
      if user is not None:
        roles = user.getRoles()
    elif userFldr.meta_type == 'Simple User Folder':
      user = self.getUserDetails(name)
      if user is not None:
        roles = user.get('roles',[])
  return roles


###################################################################################################
###################################################################################################
###
###   C l a s s   A c c e s s a b l e O b j e c t
###
###################################################################################################
###################################################################################################
class AccessableObject: 

    # -------------------------------------------------------------------------------------------
    #  AccessManager.getAuthUserPermissions:
    # -------------------------------------------------------------------------------------------
    def getAuthUserPermissions(self, REQUEST):
      auth_user = REQUEST['AUTHENTICATED_USER']
      permissions = []
      for role in list(auth_user.getRoles()):
        for p in self.permissionsOfRole(role):
          if p['selected'] and p['name'] not in permissions:
            permissions.append(p['name'])
      for ac_inherited_permission in self.ac_inherited_permissions(1):
        ac_permission = ac_inherited_permission[0]
        if ac_permission not in permissions:
          permissions.append(ac_permission)
      return permissions

    # -------------------------------------------------------------------------------------------
    #  AccessManager.getAuthUserPermittedActions:
    # -------------------------------------------------------------------------------------------
    def getAuthUserPermittedActions(self, REQUEST):
      auth_user = REQUEST['AUTHENTICATED_USER']
      permissions = self.getAuthUserPermissions(REQUEST)
      actions = []
      for ac_inherited_permission in self.ac_inherited_permissions(1):
        ac_permission = ac_inherited_permission[0]
        if ac_permission in permissions:
          ac_actions = ac_inherited_permission[1]
          actions.extend(list(ac_actions))
      return actions


###################################################################################################
###################################################################################################
###
###   C l a s s   A c c e s s M a n a g e r
###
###################################################################################################
###################################################################################################
class AccessManager(AccessableObject): 

    # Management Interface to customize access.
    manage_users = HTMLFile('dtml/access/manage_users', globals())
    
    # -------------------------------------------------------------------------------------------
    #  AccessManager.noPermission:
    # -------------------------------------------------------------------------------------------
    viewPermission__roles__ = ['ZMSAdministrator','ZMSEditor','ZMSAuthor','ZMSSubscriber']
    def viewPermission(self): pass

    # -------------------------------------------------------------------------------------------
    #  AccessManager.initRoleDefs:
    #
    #  Init Role-Definitions and Permission Settings
    # -------------------------------------------------------------------------------------------
    def initRoleDefs(self):
    
      # Init Roles.
      for role in role_defs.keys():
        role_def = role_defs[role]
        # Add Local Role.
        if not role in self.valid_roles(): self._addRole(role)
        # Set permissions for Local Role.
        self.manage_role(role_to_manage=role,permissions=role_permissions(self,role))
      
      # Clear acquired permissions.
      self.manage_acquiredPermissions([])
      
      # Grant public access.
      self.synchronizePublicAccess()


    # -------------------------------------------------------------------------------------------
    #  AccessManager.registerUserFolder:
    # -------------------------------------------------------------------------------------------
    def registerUserFolder(self):
      userFldr = self.getUserFolder()
      if userFldr is None:
        userFldr = UserFolder()
        self._setObject(userFldr.id, userFldr)
      return userFldr


    # -------------------------------------------------------------------------------------------
    #  AccessManager.unregisterUserFolder:
    # -------------------------------------------------------------------------------------------
    def unregisterUserFolder(self):
      userFldr = self.getUserFolder()
      if userFldr is not None and len(userFldr.getUsers()) == 0:
        self.manage_delObjects([userFldr.id],None)
        self.synchronizePublicAccess()


    # -------------------------------------------------------------------------------------------
    #  AccessManager.getUserFolder:
    # -------------------------------------------------------------------------------------------
    def getUserFolder(self):
      obs = self.objectValues(['User Folder','Simple User Folder'])
      if len(obs) == 1:
        return obs[0]
      else:
        return None


    # -------------------------------------------------------------------------------------------
    #  AccessManager.getUserFolders:
    # -------------------------------------------------------------------------------------------
    def getUserFolders(self):
      userFolders = []
      ob = self
      while 1:
        if ob is None: break
        try:
          userFolders.extend(ob.objectValues(['User Folder']))
          ob = ob.aq_parent
        except:
          ob = None
      return userFolders        


    # -------------------------------------------------------------------------------------------
    #  AccessManager.hasAccess:
    # -------------------------------------------------------------------------------------------
    def hasAccess(self, REQUEST):
      url = REQUEST['URL']
      auth_user = REQUEST['AUTHENTICATED_USER']
      if url.find('/manage') >= 0:
        return auth_user.has_permission('ZMS Author',self)
      else:
        return auth_user.has_permission('View',self)


    # -------------------------------------------------------------------------------------------
    #  AccessManager.__ins_user__:
    # 
    #  Creates user with specified ID.
    # -------------------------------------------------------------------------------------------
    def __ins_user__(self, name, password, confirm, roles, email, langs):
      userFldr = self.registerUserFolder()
      
      domains = []
      userFldr._addUser(name,password,confirm,roles,domains)
      user = userFldr.getUser(name)
      user.email = email
      user.langs = langs


    # -------------------------------------------------------------------------------------------
    #  AccessManager.__del_user__:
    # 
    #  Delete user with specified ID.
    # -------------------------------------------------------------------------------------------
    def __del_user__(self, name, lang, manage_lang, REQUEST):
      userFldr = self.getUserFolder()
      
      #-- Recipient
      mto = getattr(userFldr.getUser(name),'email','')
      #-- Body
      mbody = []
      mbody.append(self.getTitle(REQUEST)+' '+self.getHref2IndexHtml(REQUEST))
      mbody.append('\n')
      mbody.append('\n%s: %s (registration removed)'%(self.getLangStr('ATTR_USERID',manage_lang),name))
      mbody.append('\n')
      mbody.append('\n' + self.getLangStr('WITH_BEST_REGARDS',manage_lang))
      mbody.append('\n' + str(REQUEST['AUTHENTICATED_USER']))
      mbody.append('\n-------------------------------')
      #-- Subject
      msubject = '%s (registration removed)'%self.getDocumentElement().getTitlealt(REQUEST)
      
      # Send notification to deleted user via MailHost.
      self.sendMail(mto,msubject,''.join(mbody),REQUEST)
      
      # Delete user physically.
      userFldr._delUsers([name])


    ###############################################################################################
    #  AccessManager.manage_editUsers:
    #
    #  Change or delete users.
    ###############################################################################################
    def manage_editUsers(self, lang, manage_lang, REQUEST):
      """ AccessManager.manage_editUsers """
      message = ''
      
      # Delete.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_DELETE',manage_lang):
        if REQUEST.has_key('id'):
          name = REQUEST['id']
          self.__del_user__(name,lang,manage_lang,REQUEST)
          self.unregisterUserFolder()
          message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Change.
      # -------
      elif REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
        if REQUEST.has_key('id'):
          name = REQUEST['id']
          userFldr = self.registerUserFolder()
          user = userFldr.getUser(name)
          if user is not None:
            if REQUEST.has_key('email'):
              email = REQUEST['email'].strip()
              setattr(user,'email',email)
            if REQUEST.has_key('password') and \
               REQUEST.has_key('confirm'):
              password = REQUEST['password'].strip()
              confirm = REQUEST['confirm'].strip()
              if len(password) >= 4 and password == confirm:
                user.__ = password
            if REQUEST.has_key('langs'):
              langs = REQUEST['langs']
              if not type(langs) is type([]):
                langs = [langs]
              setattr(user,'langs',langs)
            if REQUEST.has_key('roles'):
              roles = REQUEST['roles']
              if not type(roles) is type([]):
                roles = [roles]
              if 'ZMSAdministrator' in roles:
                roles.append('Manager')
              setattr(user,'roles',roles)
            message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ###### PUBLIC ACCESS ####
      self.synchronizePublicAccess()
      
      # Return with message.
      return REQUEST.RESPONSE.redirect('manage_users?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))


    ###############################################################################################
    #  AccessManager.manage_addUser:
    #
    #  Add user.
    ###############################################################################################
    def manage_addUser(self, lang, manage_lang, REQUEST):
      """ AccessManager.manage_addUser """
      message = ''
      
      # Insert.
      # -------
      if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
      
        userFldr = self.registerUserFolder()
        name = REQUEST['_userid'].strip()
        email = REQUEST['_email'].strip()
        password = REQUEST['_password'].strip()
        confirm = REQUEST['_confirm'].strip()
        langs = REQUEST['_langs']
        if not type(langs) is type([]):
          langs = [langs]
        roles = REQUEST['_roles']
        if not type(roles) is type([]):
          roles = [roles]
        if 'ZMSAdministrator' in roles:
          roles.append('Manager')
        domains = []
        
        userFldr._addUser(name,password,confirm,roles,domains)
        user = userFldr.getUser(name)
        user.email = email
        user.langs = langs
          
        #-- Recipient
        mto = email
        #-- Body
        mbody \
          = '%s:\n%s'%(self.getDocumentElement().getTitlealt(REQUEST),self.absolute_url()) \
          + '\n%s Management-Interface (registration):\n%s/manage'%(self.getTitlealt(REQUEST),self.absolute_url()) \
          + '\n%s: %s'%(self.getLangStr('ATTR_USERID',manage_lang),name) \
          + '\n%s: %s'%(self.getLangStr('ATTR_PASSWORD',manage_lang),password) \
          + '\n%s: '%self.getLangStr('ATTR_ROLES',manage_lang)
        count = 0
        for role in roles:
          if count > 0: mbody = mbody + ', '
          mbody = mbody + self.getLangStr('ROLE_%s'%role.upper(),manage_lang)
          count = count + 1
        mbody = mbody + '\n'
        mbody = mbody + '\n' + self.getLangStr('WITH_BEST_REGARDS',manage_lang)
        mbody = mbody + '\n' + str(REQUEST['AUTHENTICATED_USER'])
        mbody = mbody + '\n-------------------------------'
        #-- Subject
        msubject = '%s (registration)'%self.getTitlealt(REQUEST)
        
        # Send notification to new user via MailHost.
        self.sendMail(mto,msubject,mbody,REQUEST)
        
        message = self.getLangStr('MSG_INSERTED',manage_lang)%self.getLangStr('ATTR_USER',manage_lang)
      
      ###### PUBLIC ACCESS ####
      self.synchronizePublicAccess()
      
      # Return with message.
      return REQUEST.RESPONSE.redirect('manage_users?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))


    ###############################################################################################
    #	AccessManager.manage_userFolderType:
    #
    #	Change type of user-folder (None, ZODB, Custom).
    ###############################################################################################
    def manage_userFolderType(self, meta_type, lang, manage_lang, REQUEST, RESPONSE):
      """ AccessManager.manage_userFolderType """
      message = ''
      
      userFldr = self.getUserFolder()
      if userFldr is not None and userFldr.meta_type != meta_type:
        for id in [userFldr.id,'addUser','deleteUser','editUser','getUserDetails','getUserNames']:
          if id in self.objectIds():
            self.manage_delObjects([id],None)
        self.synchronizePublicAccess()
          
      userFldr = self.getUserFolder()
      if userFldr is None and meta_type == 'User Folder':
        self.registerUserFolder()
      elif userFldr is None and meta_type == 'Simple User Folder':
        return RESPONSE.redirect('manage_addProduct/SimpleUserFolder/addSimpleUserFolder')
      
      # Return with message.
      return RESPONSE.redirect('manage_users?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,urllib.quote(message)))


    """
    ###############################################################################################
    #
    #  P U B L I C   A C C E S S 
    #
    ###############################################################################################
    """

    #-------------------------------------------------------------------------------------------
    #  AccessManager.hasSubscriber:
    #-------------------------------------------------------------------------------------------
    def hasSubscriber(self):
      # Check for Metadata.
      if 'attr_has_subscriber' in self.getObjAttrs().keys():
        req = {'lang':self.getPrimaryLanguage()}
        if self.getObjProperty('attr_has_subscriber',req)==1: return 1
      # Check for User with role Subscriber.
      has_subscriber = 0
      userFldr = self.getUserFolder()
      if userFldr is not None:
        for name in userFldr.getUserNames():
          roles = getRoles(self,name)
          has_subscriber = has_subscriber or 'ZMSSubscriber' in roles
      return has_subscriber

    # -------------------------------------------------------------------------------------------
    #  AccessManager.hasPublicAccess:
    # -------------------------------------------------------------------------------------------
    def hasPublicAccess(self):
      has_public_access = not self.hasSubscriber()
      parent = self.getParentNode()
      if parent is not None:
        has_public_access = has_public_access and parent.hasPublicAccess()
      return has_public_access

    # --------------------------------------------------------------------------------------------
    #  AccessManager.synchronizePublicAccess:
    #
    #  Grant / revoke public access.
    # --------------------------------------------------------------------------------------------
    def synchronizePublicAccess(self):
      if self.hasSubscriber():
        self.revokePublicAccess()
      else:
        self.grantPublicAccess()

    # -------------------------------------------------------------------------------------------
    #  AccessManager.restrictAccess:
    # -------------------------------------------------------------------------------------------
    def restrictAccess(self):
      #++ _globals.writeLog("[%s.restrictAccess]:"%self.meta_type)
      # Proxy.
      try:
        for id in ['addUser','deleteUser','editUser','getUserDetails','getUserNames','userSource']:
          if hasattr(self,id):
            ob = getattr(self,id)
            ob.manage_proxy(roles=['Manager'])
      except:
        pass
      # DTML Methods.
      for lang in self.getLanguages():
        for key in ['index','index_print','search','sitemap']:
          id = key + '_' + lang + '.html'
          if hasattr(self,id):
            ob = getattr(self,id)
            ob.manage_acquiredPermissions(permissions=['Access contents information','View'])
            for role in ['Manager']: 
              ob.manage_role(role_to_manage=role,permissions=role_permissions(ob,role))

    # -------------------------------------------------------------------------------------------
    #  AccessManager.grantPublicAccess:
    # -------------------------------------------------------------------------------------------
    def grantPublicAccess(self):
      #++ _globals.writeLog("[%s.grantPublicAccess]:"%self.meta_type)
      self.restrictAccess()
      self.manage_acquiredPermissions(role_permissions(self,'Manager'))
      for role in role_defs.keys():
        permissions = []
        if self.getLevel() == 0:
          permissions = role_permissions(self,role)
        self.manage_role(role_to_manage=role,permissions=permissions)
      # Anonymous / Authenticated.
      permissions = []
      if self.getLevel() == 0:
        permissions = ['Access contents information','View']
      self.manage_role(role_to_manage='Anonymous',permissions=permissions)
      self.manage_role(role_to_manage='Authenticated',permissions=permissions)

    # -------------------------------------------------------------------------------------------
    #  AccessManager.revokePublicAccess:
    # -------------------------------------------------------------------------------------------
    def revokePublicAccess(self):
      #++ _globals.writeLog("[%s.revokePublicAccess]:"%self.meta_type)
      self.restrictAccess()
      self.manage_acquiredPermissions([])
      for role in role_defs.keys():
        self.manage_role(role_to_manage=role,permissions=role_permissions(self,role))
      # Anonymous / Authenticated.
      permissions=['Access contents information']
      self.manage_role(role_to_manage='Anonymous',permissions=permissions)
      self.manage_role(role_to_manage='Authenticated',permissions=permissions)


###################################################################################################
#
# $Log: _accessmanager.py,v $
# Revision 1.1  2003/06/10 12:52:31  nordmann
# Initial import
#
#
