###################################################################################################
# _metacmdmanager.py
#
# $Id: _metacmdmanager.py,v 1.1 2003/06/10 12:52:31 nordmann Exp $
# $Name:  $
# $Author: nordmann $
# $Revision: 1.1 $
#
# Implementation of class MetacmdManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTML, HTMLFile
from Products.PythonScripts import PythonScript
import string
import urllib
import copy
# Product Imports.
import _globals
import _xmllib


# Example code.
# -------------

dtmlMethodExampleCode = \
  '<dtml-comment>\n' + \
  '# Example code:\n' + \
  '</dtml-comment>\n' + \
  '\n' + \
  '<dtml-call expr="REQUEST.set(\'message\',\'This is %s\'%meta_type)">\n' + \
  '<dtml-return message>\n' + \
  ''

pyScriptExampleCode = \
  '# Example code:\n' + \
  '\n' + \
  '# Import a standard function, and get the HTML request and response objects.\n' + \
  'from Products.PythonScripts.standard import html_quote\n' + \
  'request = container.REQUEST\n' + \
  'RESPONSE =  request.RESPONSE\n' + \
  '\n' + \
  '# Return a string identifying this script.\n' + \
  'print "This is the", script.meta_type, \'"%s"\' % script.getId(),\n' + \
  'if script.title:\n' + \
  '    print "(%s)" % html_quote(script.title),\n' + \
  'print "in", container.absolute_url()\n' + \
  'return printed\n' + \
  ''



# -------------------------------------------------------------------------------------------------
#  importXml
# -------------------------------------------------------------------------------------------------

def _importXml(self, item):
  id = item['id']
  # Catalog.
  meta_cmds = self.getMetaCmds()
  meta_cmd = {}
  meta_cmd['id'] = id
  meta_cmd['name'] = item['name']
  meta_cmd['description'] = item['description']
  meta_cmd['meta_types'] = item['meta_types']
  meta_cmd['roles'] = item['roles']
  meta_cmd['exec'] = item.has_key('exec') and item['exec']
  meta_cmds.append(meta_cmd)
  self.__attr_dc_metacmds__ = meta_cmds
  # Object.
  if item['meta_type'] == 'DTML Method':
    title = '*** DO NOT DELETE OR MODIFY ***'
    self.manage_addDTMLMethod(id,title)
    ob = getattr(self,id)
    ob.manage_edit(title=ob.title,data=item['data'])
  elif item['meta_type'] == 'Script (Python)':
    PythonScript.manage_addPythonScript(self,id)
    ob = getattr(self,id)
    ob.ZPythonScript_edit(params='',body=item['data'])
  return id

def importXml(self, xml):
  v = self.parseXmlString(xml)
  if type(v) is type([]):
    for item in v:
      id = _importXml(self,item)
  else:
    id = _importXml(self,v)



###################################################################################################
###################################################################################################
###
###   class MetacmdObject
###
###################################################################################################
###################################################################################################
class MetacmdObject: 

    ############################################################################################
    # MetacmdManager.manage_executeMetacmd:
    #
    # Execute Meta-Command.
    ############################################################################################
    def manage_executeMetacmd(self, custom, lang, manage_lang, REQUEST, RESPONSE):
      """ MetacmdManager.manage_executeMetacmd """
      
      message = ''
      
      for meta_cmd in self.getMetaCmds():
        print meta_cmd['name'],custom
        if meta_cmd['name'] == custom:
          ob = getattr(self,meta_cmd['id'],None)
          if ob is not None:
            if ob.meta_type == 'DTML Method':
              message = ob(self,REQUEST,RESPONSE)
            elif ob.meta_type == 'Script (Python)':
              message = ob()
      
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))



###################################################################################################
###################################################################################################
###
###   class MetacmdManager
###
###################################################################################################
###################################################################################################
class MetacmdManager: 

    # Management Interface.
    # ---------------------
    manage_customizeMetacmdForm = HTMLFile('dtml/metacmd/manage_customizeform', globals()) 


    # ------------------------------------------------------------------------------------------
    #  getMetaCmds
    # ------------------------------------------------------------------------------------------
    def getMetaCmds(self):
      if not hasattr(self,'__attr_dc_metacmds__'):
        self.__attr_dc_metacmds__ = []
      metacmds = getattr(self,'__attr_dc_metacmds__')
      for metacmd in metacmds:
        if not metacmd.has_key('exec'):
          metacmd['exec'] = 1
      return metacmds
    
        
    ############################################################################################
    # MetacmdManager.manage_changeMetacmds:
    #
    # Change Meta-Commands.
    ############################################################################################
    def manage_changeMetacmds(self, btn, lang, manage_lang, REQUEST, RESPONSE):
        """ MetacmdManager.manage_changeMetacmds """

        message = ''
        id = ''
    
        # Insert.
        # -------
        if btn == self.getLangStr('BTN_INSERT',manage_lang):
          id = REQUEST['_id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          meta_cmd = {}
          meta_cmd['id'] = id
          meta_cmd['name'] = REQUEST.get('_name','')
          meta_cmd['description'] = ''
          meta_cmd['meta_types'] = []
          meta_cmd['roles'] = ['ZMSAdministrator']
          meta_cmd['exec'] = REQUEST.get('_exec',0)
          meta_cmds.append(meta_cmd)
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          if REQUEST.get('_type','DTML Method') == 'DTML Method':
            title = '*** DO NOT DELETE OR MODIFY ***'
            self.manage_addDTMLMethod(id,title)
            ob = getattr(self,id)
            ob.manage_edit(title=ob.title,data=dtmlMethodExampleCode)
          elif REQUEST.get('_type','DTML Method') == 'Script (Python)':
            PythonScript.manage_addPythonScript(self,id)
            ob = getattr(self,id)
            ob.ZPythonScript_edit(params='',body=pyScriptExampleCode)
            
        # Delete.
        # -------
        elif btn == self.getLangStr('BTN_DELETE',manage_lang):
          id = REQUEST['id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          for meta_cmd in meta_cmds:
            if meta_cmd['id'] == id:
              meta_cmds.remove(meta_cmd)
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          self.manage_delObjects(ids=[id])
          # Message.
          id = ''
          message = self.getLangStr('MSG_CHANGED',manage_lang)

        # Change.
        # -------
        elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
          id = REQUEST['el_id']
          old_id = REQUEST['id']
          # Catalog.
          meta_cmds = self.getMetaCmds()
          for meta_cmd in meta_cmds:
            if meta_cmd['id'] == old_id:
              meta_cmd['id'] = id
              meta_cmd['name'] = REQUEST.get('el_name','')
              meta_cmd['description'] = REQUEST.get('el_description','')
              meta_cmd['meta_types'] = REQUEST.get('el_meta_types',[])
              meta_cmd['roles'] = REQUEST.get('el_roles',[])
              meta_cmd['exec'] = REQUEST.get('el_exec',0)
              meta_cmd = meta_cmd.copy()
          self.__attr_dc_metacmds__ = meta_cmds
          # Object.
          if id != old_id:
            self.manage_renameObject(id=old_id,new_id=id)
          ob = getattr(self,id)
          if ob.meta_type == 'DTML Method':
            ob.manage_edit(title=ob.title,data=REQUEST.get('el_data',''))
          elif ob.meta_type == 'Script (Python)':
            ob.ZPythonScript_edit(params='',body=REQUEST.get('el_data',''))

        # Import.
        # -------
        elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
          f = REQUEST['file']
          importXml(self,xml=f)
          message = self.getLangStr('MSG_IMPORTED',manage_lang)%('<i>%s</i>'%f.filename)

        # Export.
        # -------
        elif btn == self.getLangStr('BTN_EXPORT',manage_lang):
          value = []
          ids = REQUEST.get('ids',[])
          metaCmds = self.getMetaCmds()
          for metaCmd in metaCmds:
            id = metaCmd['id']
            if id in ids or len(ids) == 0:
              # Catalog.
              el_name = metaCmd['name']
              el_description = metaCmd['description']
              el_meta_types = metaCmd['meta_types']
              el_roles = metaCmd['roles']
              el_exec = metaCmd['exec']
              # Object.
              ob = getattr(self,id)
              el_meta_type = ob.meta_type
              if ob.meta_type == 'DTML Method':
                el_data = ob.raw
              else:
                el_data = ob.body()
              # Value.
              value.append({'id':id,'name':el_name,'description':el_description,'meta_types':el_meta_types,'roles':el_roles,'exec':el_exec,'meta_type':el_meta_type,'data':el_data})
          # XML.
          if len(value)==1:
            value = value[0]
          content_type = 'text/xml'
          filename = 'specCmd.xml'
          export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
          export = export + self.toXmlString(value,1)
          RESPONSE.setHeader('Content-Type',content_type)
          RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
          return export

        # Return with message.
        message = urllib.quote(message)
        return RESPONSE.redirect('manage_customizeMetacmdForm?lang=%s&manage_lang=%s&manage_tabs_message=%s&id=%s'%(lang,manage_lang,message,id))

###################################################################################################
#
# $Log: _metacmdmanager.py,v $
# Revision 1.1  2003/06/10 12:52:31  nordmann
# Initial import
#
#
