###################################################################################################
# _versionmanager.py
#
# $Id: _versionmanager.py,v 1.1 2003/06/10 12:52:31 nordmann Exp $
# $Name:  $
# $Author: nordmann $
# $Revision: 1.1 $
#
# Implementation of class VersionManager (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import copy
import sys
import time
import urllib
# Product Imports.
import _globals 
import _blobfields 
import _zmsattributecontainer


# DEPRECATED: Products.zms._myobjects.MyFile
class MyFile(_blobfields.MyFile): pass
# DEPRECATED: Products.zms._myobjects.MyImage
class MyImage(_blobfields.MyImage): pass


# Object-States.
# --------------
STATE_NEW = 0
STATE_MODIFIED = 1
STATE_MODIFIED_SILENT = 2
STATE_DELETED = 3
STATE_REQUESTED = 4
STATE_ROLLEDBACK = 5
STATE_REJECTED = 6
STATES_WORKFLOW = [STATE_REQUESTED,STATE_REJECTED,STATE_ROLLEDBACK]
STATES_SILENT = [STATE_MODIFIED_SILENT,STATE_REQUESTED,STATE_REJECTED,STATE_ROLLEDBACK]


# ---------------------------------------------------------------------------------------------
#  _versionmanager.delObjStates:
#
#  Deletes object-state of given object.
# ---------------------------------------------------------------------------------------------
def delObjStates(self, obj_states=[], REQUEST={}):
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  states = self.getObjStatesRaw()
  for obj_state in obj_states:
    if obj_state in states:
      del states[states.index(obj_state)]
    if getObjStateName(obj_state,lang) in states:
      del states[states.index(getObjStateName(obj_state,lang))]
  self.__work_state__.states = copy.deepcopy(states)
  self.__work_state__ = copy.deepcopy(self.__work_state__)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.resetObjState:
#
#  Resets work-state of given object.
# ---------------------------------------------------------------------------------------------
def resetObjState(self, REQUEST):
  delObjStates(self,['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED','STATE_REQUESTED','STATE_ROLLEDBACK','STATE_REJECTED'],REQUEST)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.setWorkflowEventBy:
#
#  Applies information about user-id and date of workflow-event.
# ---------------------------------------------------------------------------------------------
def setWorkflowEventBy(self, REQUEST):
  # Set Properties.
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  auth_user = REQUEST.get('AUTHENTICATED_USER','Anonymous User')
  self.setObjProperty('work_uid',str(auth_user),lang)
  self.setObjProperty('work_dt',time.time(),lang)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.setChangedBy:
#
#  Applies information about user-id and date of change.
# ---------------------------------------------------------------------------------------------
def setChangedBy(self, REQUEST, reset=1):
  # Add Version.
  if reset and self.getConfProperty('ZMS.Version.active',0)==1:
    req = {'lang':'*'}
    workAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
    self.cloneObjAttrs(getattr(self,self.version_work_id),workAttrCntnr,req)
    self.version_work_id = workAttrCntnr.id
  # Set Properties.
  lang = REQUEST.get('lang',self.getPrimaryLanguage())
  auth_user = _globals.nvl(REQUEST.get('AUTHENTICATED_USER',None),'Anonymous User')
  self.setObjProperty('change_lang',lang)
  self.setObjProperty('change_action',REQUEST.get('URL','/').split('/')[-1])
  self.setObjProperty('change_uid',str(auth_user),lang)
  self.setObjProperty('change_dt',time.localtime(time.time()),lang)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.setObjState:
#
#  Applies new state.
# ---------------------------------------------------------------------------------------------
def setObjState(self, state, lang):
  state = getObjStateName(state,lang)
  states = self.getObjStatesRaw()
  if not state in states:
    states.append(state)
  self.__work_state__.states = copy.deepcopy(states)
  self.__work_state__ = copy.deepcopy(self.__work_state__)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.getObjStateName:
#
#  Returns name object state in given language.
# ---------------------------------------------------------------------------------------------
def getObjStateName(state, lang):
  return '%s_%s'%(state,lang.upper())


# ---------------------------------------------------------------------------------------------
#  _versionmanager.doPack:
#
#  Remove unused versions.
# ---------------------------------------------------------------------------------------------
def doPack(self, REQUEST): 

  ##### Pack (remove unused versions) ####
  now = time.localtime(time.time())
  period = self.getConfProperty('ZMS.Version.autopack','')
  ids = []
  for ob in self.getObjVersions():
    change_dt = ob.getObjProperty('change_dt',REQUEST)
    append = 1
    append = append and (ob.id not in [self.version_live_id,self.version_work_id])
    append = append and (len(period)==0 or change_dt is None or (time.mktime(_globals.stripDateTime(now))-time.mktime(_globals.stripDateTime(change_dt)))/(24*60*60)>=int(period))
    if append:
      ids.append(ob.id)
  self.manage_delObjects(ids=ids)
  
  ##### Process child-objects ####
  for ob in self.getChildNodes():
    doPack(ob,REQUEST)


# ---------------------------------------------------------------------------------------------
#  _versionmanager.doAutocommit:
#
#  Commit pending changes of all objects.
# ---------------------------------------------------------------------------------------------
def doAutocommit(self, REQUEST): 
  #++ print "[%s.doAutocommit]"%self.meta_type
  
  ##### Auto-Commit ####
  lang = REQUEST['lang']
  for s_lang in self.getLanguages():
    REQUEST.set('lang',s_lang)
    self.onChangeObj(REQUEST,forced=1)
  REQUEST.set('lang',lang)
  
  ##### Process child-objects ####
  for ob in self.getChildNodes():
    doAutocommit(ob,REQUEST)


###################################################################################################
###################################################################################################
###
###   C l a s s   V e r s i o n M a n a g e r :
###
###################################################################################################
###################################################################################################
class VersionManager: 


    # Management Interface.
    # ---------------------
    version_input_fields = HTMLFile('dtml/versionmanager/version_input_fields', globals())
    version_object_state = HTMLFile('dtml/versionmanager/version_object_state', globals())


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.resetObjVersion:
    #
    #  Resets work-version of object-attributes.
    # ---------------------------------------------------------------------------------------------
    def resetObjVersion(self):
      # States.
      self.__work_state__ = _globals.MyClass()
      # Versions.
      if len(self.getObjVersions())==0:
        req = {'lang':'*'}
        liveAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
        self.cloneObjAttrs(self,liveAttrCntnr,req)
        self.version_live_id = liveAttrCntnr.id
        workAttrCntnr = _zmsattributecontainer.manage_addZMSAttributeContainer(self)
        self.cloneObjAttrs(liveAttrCntnr,workAttrCntnr,req)
        self.version_work_id = workAttrCntnr.id


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjVersion:
    #
    #  Live or work-version of object-attributes.
    # ---------------------------------------------------------------------------------------------
    def getObjVersion(self, REQUEST={}):
      ob = None
      id = None
      if REQUEST.get('ZMS_VERSION',None) is not None:
        if REQUEST['ZMS_VERSION'].find(self.id+'/')==0:
          id = REQUEST['ZMS_VERSION'][len(self.id+'/'):]
      if id is None:
        if _globals.isPreviewRequest(REQUEST):
          id = self.version_work_id
        else:
          id = self.version_live_id
      ob = getattr(self,id)
      s = ob.id # Never delete this line!
      return ob


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.getObjVersions:
    # ---------------------------------------------------------------------------------------------
    def getObjVersions(self, REQUEST={}):
      obs = []
      for ob in self.objectValues(['ZMSAttributeContainer']):
        lang = REQUEST.get('lang','*')
        change_lang = ob.getObjProperty('change_lang',REQUEST)
        if (lang=='*') or \
           (lang!='*' and ob.id!=self.version_live_id and \
            ((len(obs)==0) or \
             (change_lang==lang) or \
             (change_lang=='' and lang==self.getPrimaryLanguage()))):
          obs.insert(0,ob)
      return obs


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.isCommitted:
    #
    #  Checks if object can by displayed.
    # ---------------------------------------------------------------------------------------------
    def isCommitted(self, REQUEST):
      lang = REQUEST.get('lang',self.getPrimaryLanguage())
      preview = REQUEST.get('preview','') == 'preview'
      obj_states = self.getObjStatesRaw()
      committed = 0
      committed = committed or (not preview and not getObjStateName('STATE_NEW',lang) in obj_states)
      committed = committed or (preview and not getObjStateName('STATE_DELETED',lang) in obj_states)
      return committed


    """
    ###############################################################################################
    #
    #  Object State
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    # 	IN (Object State Names)
    # ---------------------------------------------------------------------------------------------
    def inObjStates(self, states, REQUEST):
      obj_state_names = []
      obj_state_names.extend(self.getObjStatesRaw())
      obj_state_names.extend(self.getObjStateNames(REQUEST))
      if len(obj_state_names) > 0:
        lang = REQUEST.get('lang',self.getPrimaryLanguage())
        for state in states:
          obj_state_name = getObjStateName(state,lang)
          if obj_state_name in obj_state_names:
            return 1
      return 0


    # ---------------------------------------------------------------------------------------------
    # 	getObjStatesRaw
    # ---------------------------------------------------------------------------------------------
    def getObjStatesRaw(self):
      if not hasattr(self,'__work_state__'):
        self.__work_state__ = _globals.MyClass()
      return getattr(self.__work_state__,'states',[])


    # ---------------------------------------------------------------------------------------------
    # 	GET (Object State Names)
    # ---------------------------------------------------------------------------------------------
    def getObjStateNames(self, REQUEST):
    
      states = self.getObjStatesRaw()

      lang = REQUEST.get('lang',None)
          
      if lang is None:
        obj_states = states
      else:
        obj_states = []
        
        for state in ['STATE_NEW','STATE_MODIFIED','STATE_MODIFIED_SILENT','STATE_DELETED']:
          obj_state_name = getObjStateName(state,lang)
          if obj_state_name in states:
            obj_states.append(obj_state_name)
      
      # Special Objects / Link-Containers.
      children = []
      if self.meta_type == 'ZMSCustom':
        children = self.getObjChildren('',REQUEST)
      elif self.meta_type == 'ZMSLinkContainer':
        children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
      for child in children:
        for child_obj_state in child.getObjStateNames(REQUEST):
          lang = child_obj_state[child_obj_state.rfind('_')+1:].lower()
          state = 'STATE_MODIFIED'
          obj_state_name = getObjStateName(state,lang)
          if not obj_state_name in obj_states:
            obj_states.append(obj_state_name)

      # Return Object State Names.
      return obj_states


    # ---------------------------------------------------------------------------------------------
    # 	VersionManager.isOnlyModifiedSilent:
    #
    #	Returns true if object is only modified silent.
    # ---------------------------------------------------------------------------------------------
    def isOnlyModifiedSilent(self):
      silent = 1
      states = self.getObjStatesRaw()
      for state in states:
        silent = silent and \
                 (state.find('STATE_MODIFIED_SILENT') == 0 or 
                  state.find('STATE_REQUESTED') == 0 or
                  state.find('STATE_REJECTED') == 0 or 
                  state.find('STATE_ROLLEDBACK') == 0)
      return silent

    # ---------------------------------------------------------------------------------------------
    # 	SET (Modified Silent)
    # ---------------------------------------------------------------------------------------------
    def setObjStateModifiedSilent(self, REQUEST):
      setObjState(self,'STATE_MODIFIED_SILENT',REQUEST['lang'])
      self.clearWorkflowEvents(REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	SET (Modified)
    # ---------------------------------------------------------------------------------------------
    def setObjStateModified(self, REQUEST):
      setChangedBy(self,REQUEST)
      setObjState(self,'STATE_MODIFIED',REQUEST['lang'])
      self.clearWorkflowEvents(REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	SET (Deleted)
    # ---------------------------------------------------------------------------------------------
    def setObjStateDeleted(self, REQUEST):
      setChangedBy(self,REQUEST)
      setObjState(self,'STATE_DELETED',REQUEST['lang'])
      self.clearWorkflowEvents(REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	SET (New)
    # ---------------------------------------------------------------------------------------------
    def setObjStateNew(self, REQUEST, reset=1):
      if reset: self.resetObjVersion()
      setChangedBy(self,REQUEST,reset)
      setObjState(self,'STATE_NEW',REQUEST['lang'])
      self.clearWorkflowEvents(REQUEST)



    """
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.onChangeObj:
    # ---------------------------------------------------------------------------------------------
    def onChangeObj(self, REQUEST, forced=0):
      #++ if _globals.debug: print '[%s.onChangeObj]'%(self.meta_type)
      
      ##### Autocommit ####
      if self.getAutocommit():
        parent = self.getParentNode()
        delete = self.commitObjChanges(parent,REQUEST,forced)
        if delete: 
          parent.normalizeSortIds()
      
      ##### Access ####
      self.synchronizePublicAccess()
      
      ##### Catalog ####
      self.synchronizeSearch(REQUEST)
      
      ##### Cache ####
      self.synchronizeCachePage(REQUEST)


    # ---------------------------------------------------------------------------------------------
    #  VersionManager.commitObjChanges
    # ---------------------------------------------------------------------------------------------
    def commitObjChanges(self, parent, REQUEST, forced=0):
      if _globals.debug: print '[%s.commitObjChanges]: %s'%(self.meta_type,self.id)
      delete = 0
      
      ##### Trigger custom onChangeObj-Event (if there is one) ####
      try: 
        if self.meta_type == 'ZMSCustom':
          metaObjAttr = self.getMetaobjAttr(self.meta_id,'onChangeObjEvt')
          if metaObjAttr is not None:
            v = self.getObjProperty('onChangeObjEvt',REQUEST)
      except:
        _globals.writeLog("[%s.onChangeObjEvt]: id=%s"%(self.meta_type,self.id))
        _globals.writeLog("Type: %s"%str(sys.exc_type))
        _globals.writeLog("Value: %s"%str(sys.exc_value))
        _globals.writeLog("Traceback: %s"%str(sys.exc_traceback))
      try: 
        if hasattr(self,'onChangeObjEvt'):
          v = getattr(self,'onChangeObjEvt')(context=self,REQUEST=REQUEST)
      except:
        _globals.writeLog("[%s.onChangeObjEvt]: id=%s"%(self.meta_type,self.id))
        _globals.writeLog("Type: %s"%str(sys.exc_type))
        _globals.writeLog("Value: %s"%str(sys.exc_value))
        _globals.writeLog("Traceback: %s"%str(sys.exc_traceback))
      
      ##### Commit delete object. ####
      if self.inObjStates(['STATE_DELETED'],REQUEST):
        if self.id in parent.objectIds():
          parent.moveObjToTrashcan(self,REQUEST)
          resetObjState(self,REQUEST)
          delete = 1
          
      ##### Commit modifications. ####
      elif self.getObjStateNames(REQUEST) or forced:
        self.cloneObjAttrs(getattr(self,self.version_work_id),getattr(self,self.version_live_id),REQUEST)
        # Special Objects / Link-Containers.
        children = []
        if self.meta_type == 'ZMSCustom':
          children = self.getObjChildren('',REQUEST)
        elif self.meta_type == 'ZMSLinkContainer':
          children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
        for child in children:
          child.commitObjChanges(self,REQUEST)
        # Reset object-state.
        resetObjState(self,REQUEST)
        
      # Return flag for deleted objects.
      return delete


    """
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager.rollbackObjChanges
    # ---------------------------------------------------------------------------------------------
    def rollbackObjChanges(self, parent, REQUEST):
      delete = 0
      user = REQUEST['AUTHENTICATED_USER']
      if str(user) == self.getObjProperty('change_uid',REQUEST) or \
         user.has_role('Manager') or \
         user.has_role('ZMSAdministrator') or \
         user.has_role('ZMSEditor'):
        ##### Rollback insert object. ####
        if self.inObjStates(['STATE_NEW'],REQUEST):
          if self.id in parent.objectIds():
            parent.moveObjToTrashcan(self,REQUEST)
            resetObjState(self,REQUEST)
            delete = 1
        ##### Rollback modifications. ####
        elif self.getObjStateNames(REQUEST):
          setChangedBy(self,REQUEST)
          self.cloneObjAttrs(getattr(self,self.version_live_id),getattr(self,self.version_work_id),REQUEST)
          # Special Objects / Link-Containers.
          children = []
          if self.meta_type == 'ZMSCustom':
            children = self.getObjChildren('',REQUEST)
          elif self.meta_type == 'ZMSLinkContainer':
            children = self.getChildNodes(REQUEST,['ZMSLinkElement'])
          for child in children:
            child.rollbackObjChanges(self,REQUEST)
          # Reset object-state.
          resetObjState(self,REQUEST)
      # Return flag for deleted objects.
      return delete


    """
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManager._requestVersionChanges
    # ---------------------------------------------------------------------------------------------
    def _requestVersionChanges(self, parent, lang, manage_lang, REQUEST):
      s = ''
      if len(self.getObjStateNames(REQUEST)) > 0:
        s = s + '\n* %s: '%self.display_type(REQUEST) 
        for state in ['STATE_NEW','STATE_MODIFIED','STATE_DELETED']:
          if self.inObjStates([state],REQUEST):
            s = s + '[%s] '%self.getLangStr(state,manage_lang)
        s = s + '%s %s '%(self.getLangStr('ON',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'DATE_FMT'))
        s = s + '%s %s '%(self.getLangStr('AT',manage_lang),self.getLangFmtDate(self.getObjProperty('change_dt',REQUEST),manage_lang,'TIME_FMT'))
        s = s + '%s %s '%(self.getLangStr('BY',manage_lang),self.getObjProperty('change_uid',REQUEST))
      return s


    """
    ###############################################################################################
    #
    #   V E R S I O N
    #
    ###############################################################################################
    """

    ###############################################################################################
    #  VersionManager.manage_UndoVersion:
    #
    #  Undo version changes.
    ###############################################################################################
    manage_UndoVersionForm = HTMLFile('dtml/versionmanager/manage_undoversionform', globals())
    def manage_UndoVersion(self, lang, manage_lang, REQUEST):
      """ VersionManager.manage_UndoVersion """
      message = ''
      
      id = REQUEST.get('id','')
      attrCntnr = getattr(self,id,None)
      if attrCntnr is not None:
        # Restore attributes.
        self.setObjStateModified(REQUEST)
        self.cloneObjAttrs(attrCntnr,getattr(self,self.version_work_id),REQUEST)
        self.onChangeObj(REQUEST)
        # Restore sort-order.
        parent = self.getParentNode()
        if parent is not None: parent.normalizeSortIds(_globals.id_prefix(self.id))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return REQUEST.RESPONSE.redirect('manage_UndoVersionForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))



###################################################################################################
###################################################################################################
###
###   C l a s s   V e r s i o n M a n a g e r C o n t a i n e r:
###
###################################################################################################
###################################################################################################
class VersionManagerContainer: 

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.clearWorkflowEvents:
    #
    #  Resets information about workflow-events.
    # ---------------------------------------------------------------------------------------------
    def clearWorkflowEvents(self, REQUEST):
      lang = REQUEST['lang']
      self.setObjProperty('work_uid',None,lang)
      self.setObjProperty('work_dt',None,lang)
      # Resets information about workflow-events.
      delObjStates(self,['STATE_REQUESTED','STATE_ROLLEDBACK','STATE_REJECTED'],REQUEST)
      
    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.normalizeSortIdsAfterDelete:
    #
    #  Resets child-objects after delete..
    # ---------------------------------------------------------------------------------------------
    def normalizeSortIdsAfterDelete(self):
      self.normalizeSortIds()
      # Resets information about silent-modifications.
      for child in self.getChildNodes():
        delObjStates(child,['STATE_MODIFIED_SILENT'])

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.isObjModified:
    # ---------------------------------------------------------------------------------------------
    def isObjModified(self, REQUEST):
      state = self.inObjStates(['STATE_NEW','STATE_MODIFIED','STATE_DELETED'],REQUEST)
      for child in self.getChildNodes(REQUEST):
        state = state + child.inObjStates(['STATE_NEW','STATE_MODIFIED','STATE_DELETED'],REQUEST)
      return state


    """
    ###############################################################################################    
    #
    #   T A S K S 
    #
    ###############################################################################################    
    """

    # Management Interface.
    manage_tasks = HTMLFile('dtml/versionmanager/manage_tasks', globals()) 
    tasklist0 = HTMLFile('dtml/versionmanager/tasklist0', globals()) 
    tasklist1 = HTMLFile('dtml/versionmanager/tasklist1', globals()) 
    tasklist2 = HTMLFile('dtml/versionmanager/tasklist2', globals()) 
    tasklist3 = HTMLFile('dtml/versionmanager/tasklist3', globals()) 
    tasklist4 = HTMLFile('dtml/versionmanager/tasklist4', globals()) 
    tasklist5 = HTMLFile('dtml/versionmanager/tasklist5', globals()) 

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.getTasks:
    # ---------------------------------------------------------------------------------------------
    def getTasks(self, task_type, REQUEST):
      rtn = []
      append = 1
      if task_type == 0:
        append = append and self.isObjModified(REQUEST)
      elif task_type == 1:
        append = append and self.inObjStates(['STATE_REQUESTED'],REQUEST)
      elif task_type == 2:
        append = append and self.inObjStates(['STATE_REJECTED'],REQUEST)
      elif task_type == 4:
        translate = len(self.getObjProperty('change_uid',REQUEST)) == 0
        for child in self.getChildNodes(REQUEST):
          translate = translate or len(child.getObjProperty('change_uid',REQUEST)) == 0
        append = append and self.getDCCoverage(REQUEST) not in self.getLanguages() and translate
      if append: rtn.append(self)
      for child in self.getChildNodes(REQUEST,self.PAGES):
        rtn.extend(child.getTasks(task_type,REQUEST))
      return rtn


    """
    ###############################################################################################
    #
    #   C O M M I T
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.doAutocommitIfOnlyModifiedSilent:
    #
    #  Auto commit container if it is not modified and nothing but silently
    #  modified objects (sort_id).
    # ---------------------------------------------------------------------------------------------
    def doAutocommitIfOnlyModifiedSilent(self):
      #++ if _globals.debug: print '[%s.doAutocommitIfOnlyModifiedSilent]: %s'%(self.meta_type,self.absolute_url())
      req = {'lang':self.getPrimaryLanguage(),'preview':'preview'}
      flag = self.isOnlyModifiedSilent()
      for child in self.getChildNodes(req):
        flag = flag and child.isOnlyModifiedSilent()
      flag = flag or self.getAutocommit()
      if flag:
        self.commitObj()


    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.commitObj
    # ---------------------------------------------------------------------------------------------
    def commitObj(self, REQUEST={}):
      if _globals.debug: print "[%s.commitObj]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child.commitObjChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      parent = self.getParentNode()        
      delete = self.commitObjChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url


    ###############################################################################################
    #  VersionManagerContainer.manage_commitVersionChanges:	
    #
    #  Commit version changes.
    ###############################################################################################
    def manage_commitVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_commitVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      ob = self
      while ob is not None:
        userFldr = ob.getUserFolder()
        if userFldr is not None:
          user = userFldr.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Commit ####
      url = self.commitObj(REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_COMMITVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """
    ###############################################################################################
    #
    #   R O L L B A C K
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  VersionManagerContainer.rollbackObj
    # ---------------------------------------------------------------------------------------------
    def rollbackObj(self, REQUEST):
      #++ if _globals.debug: print "[%s.rollbackObj]: id=%s"%(self.meta_type,self.id)
      ##### Children ####
      delete = 0
      for child in self.getChildNodes():
        delete = delete + child.rollbackObjChanges(self,REQUEST)
      if delete: self.normalizeSortIdsAfterDelete()
      ##### Self ####
      parent = self.getParentNode()
      delete = self.rollbackObjChanges(parent,REQUEST)
      url = self.absolute_url()
      if delete: 
        parent.normalizeSortIdsAfterDelete()
        url = parent.absolute_url()
      # Return new URL.
      return url


    ###############################################################################################
    #  VersionManagerContainer.manage_rejectVersionChanges:
    #
    #  Reject version changes.
    ###############################################################################################
    def manage_rejectVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rejectVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      ob = self
      while ob is not None:
        userFldr = ob.getUserFolder()
        if userFldr is not None and userFldr.meta_type=='User Folder':
          user = userFldr.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Set object state ####
      self.clearWorkflowEvents(REQUEST)
      setObjState(self,'STATE_REJECTED',lang)
      setWorkflowEventBy(self,REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_REJECTVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    ###############################################################################################
    #  VersionManagerContainer.manage_rollbackVersionChanges:	
    #
    #  Rollback version changes.
    ###############################################################################################
    def manage_rollbackVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_rollbackVersionChanges """
      
      ##### Notification ####
      
      #-- Recipient
      mto = ''
      name = self.getObjProperty('work_uid',REQUEST)
      ob = self
      while ob is not None:
        userFldr = ob.getUserFolder()
        if userFldr is not None and userFldr.meta_type=='User Folder':
          user = userFldr.getUser(name)
          if user is not None:
            mto = getattr(user,'email','')
        ob = ob.getParentNode()
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      #-- Body
      mbody \
        = self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url()) \
        + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      # Send notification via MailHost.
      self.sendMail(mto,msubject,mbody,REQUEST)
      
      ##### Rollback ####
      url = self.rollbackObj(REQUEST)
      
      ##### Set object state ####
      self.clearWorkflowEvents(REQUEST)
      setObjState(self,'STATE_ROLLEDBACK',lang)
      setWorkflowEventBy(self,REQUEST)
      self.doAutocommitIfOnlyModifiedSilent()
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_ROLLBACKVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(url,lang,manage_lang,message))


    """
    ###############################################################################################
    #
    #   R E Q U E S T
    #
    ###############################################################################################
    """

    ###############################################################################################
    #  VersionManagerContainer.manage_requestVersionChanges:
    #
    #  Request commit of version changes.
    ###############################################################################################
    def manage_requestVersionChanges(self, lang, manage_lang, REQUEST):
      """ VersionManagerContainer.manage_requestVersionChanges """
      
      ##### Set object state ####
      self.clearWorkflowEvents(REQUEST)
      setObjState(self,'STATE_REQUESTED',lang)
      setWorkflowEventBy(self,REQUEST)
      
      ##### Notifications ####
      
      #-- Recipients
      recipients = []
      role = 'ZMSEditor'
      if REQUEST['AUTHENTICATED_USER'].has_role('ZMSEditor'):
        role = 'ZMSAdministrator'
      ob = self
      while ob is not None:
        userFldr = ob.getUserFolder()
        if userFldr is not None and userFldr.meta_type=='User Folder':
          for user in userFldr.getUsers():
            if user.has_role(role) and \
               len(getattr(user,'email','')) > 0 and \
               lang in getattr(user,'langs',[]):
              recipients.append({'userid':str(user),'email':getattr(user,'email',None)})
        ob = ob.getParentNode()
      
      #-- Body
      
      # Init message body.
      mbody \
        = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),str(REQUEST['AUTHENTICATED_USER'])) \
        + '\n%s/manage'%(self.absolute_url())
      # Build message body.
      mbody = mbody + self._requestVersionChanges(self,lang,manage_lang,REQUEST)
      mbody = mbody + '\n'
      for child in self.getChildNodes():
        mbody = mbody + child._requestVersionChanges(self,lang,manage_lang,REQUEST)
      # With best regards.                
      mbody = mbody + '\n\n%s\n%s'%(self.getLangStr('WITH_BEST_REGARDS',manage_lang),str(REQUEST['AUTHENTICATED_USER']))
      
      #-- Subject
      msubject \
        = self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang) \
        + ' %s "%s"'%(self.display_type(REQUEST),self.getTitle(REQUEST)) \
        + ' %s %s '%(self.getLangStr('BY',manage_lang),REQUEST['AUTHENTICATED_USER'])
      
      for recipient in recipients:
        
        #-- Recipient
        mto = recipient['email']
        
        # Send notification via MailHost.
        self.sendMail(mto,msubject,mbody,REQUEST)
      
      # Return with message.
      message = urllib.quote(self.getLangStr('ACTION_MANAGE_REQUESTVERSIONCHANGES',manage_lang))
      return REQUEST.RESPONSE.redirect('manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))

###################################################################################################
#
# $Log: _versionmanager.py,v $
# Revision 1.1  2003/06/10 12:52:31  nordmann
# Initial import
#
#
