###################################################################################################
# zms.py
#
# $Id: zms.py,v 1.1 2003/06/10 12:52:31 nordmann Exp $
# $Name:  $
# $Author: nordmann $
# $Revision: 1.1 $
#
# Implementation of class ZMS (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile, ImageFile
from DateTime.DateTime import DateTime
from AccessControl.User import UserFolder
from App.Common import package_home
from OFS.Image import Image
from sys import *
import copy
import string 
import time
import urllib
# Product imports.
import _sequence
import _mediadb
import _builder
import _cachemanager
import _globals
import _fileutil
import _ftpmanager
import _importable
import _metadictmanager
import _metacmdmanager
import _metaobjmanager
import _metapatternmanager
import _multilangmanager
import _printablemanager
import _objattrs
import _textformatmanager
import _versionmanager
import _xmllib
import _xsllib
import _zcatalogmanager
import zmscontainerobject
from zmscustom import ZMSCustom
from zmsdocument import ZMSDocument
from zmsfile import ZMSFile
from zmsgraphic import ZMSGraphic
from zmslinkcontainer import ZMSLinkContainer
from zmslinkelement import ZMSLinkElement
from zmsnote import ZMSNote
from zmsrubrik import ZMSRubrik
from zmssqldb import ZMSSqlDb
from zmssysfolder import ZMSSysFolder
from zmstable import ZMSTable
from zmsteasercontainer import ZMSTeaserContainer
from zmsteaserelement import ZMSTeaserElement
from zmstextarea import ZMSTextarea
from zmstrashcan import ZMSTrashcan


###################################################################################################
###################################################################################################
###   
###   C o m m o n   F u n c t i o n ( s )
###   
###################################################################################################
###################################################################################################

# -------------------------------------------------------------------------------------------------
#  _getNewId
# -------------------------------------------------------------------------------------------------
def _getNewId(_obj, _id):
  for obj in _obj.objectValues(_obj.dGlobalAttrs.keys()):
    try:
      sObjId = str(obj.id)
      iObjId = sObjId[1:].atoi()
      if iObjId > _id:
        _id = iObjId
      newid = _getNewId(obj,_id)
      if newid > _id:
        _id = newid
    except:
      pass
  return _id


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSForm = HTMLFile('manage_addzmsform', globals()) 
def manage_addZMS(self, lang, manage_lang, REQUEST, RESPONSE):
  """ manage_addZMS """
  
  if REQUEST['btn'] == 'Add':
    
    ##### Add TemplateSet ####
    # Import theme from ZEXP.
    theme = REQUEST['theme']
    theme_zexp = theme + '.zexp'
    _fileutil.importZexp(self,package_home(globals())+'/import/',theme_zexp)
    # Assign folder-id.
    folder_id = REQUEST['folder_id']
    self.manage_renameObject(theme,folder_id)
    homeElmnt = getattr(self,folder_id)
      
    ##### Add Zope-Manager ####
    if not homeElmnt.objectValues(['User Folder']):
      userFldr = UserFolder()
      homeElmnt._setObject(userFldr.id, userFldr)
    name = REQUEST.get('manager_name','')
    password = REQUEST.get('manager_password','')
    confirm = password
    roles = ['Manager']
    domains = []
    userFldr._addUser(name,password,confirm,roles,domains)
        
    ##### Create ####
    id = 'content'
    obj = ZMS(id)
    homeElmnt._setObject(obj.id, obj)
    obj = getattr(homeElmnt,obj.id)
        
    ### Init Configuration.
    obj.setConfProperty('ZMS.autocommit',1)
    obj.setConfProperty('ZMS.Version.active',0)
    obj.setConfProperty('ZMS.Version.autopack',2)
    obj.setConfProperty('ZMSAdministrator.email',REQUEST.get('manager_email',''))
    obj.setConfProperty('ASP.ip_or_domain',REQUEST.get('asp_ip_or_domain',''))
        
    ### Init StyleSheet.
    defaultCSS = ''
    if 'common' in homeElmnt.objectIds(['Folder']):
      if 'defaultCSS' in homeElmnt.common.objectIds(['DTML Method']):
        defaultCSS = homeElmnt.common.defaultCSS.raw
        homeElmnt.common.manage_delObjects(ids=['defaultCSS'])
    obj.getStylesheet(defaultCSS)
        
    ### Init ZCatalog.
    obj.setCatalogPeriod()
    obj.recreateCatalog()
        
    ### Init Languages.
    obj.setPrimaryLanguage(lang)
    obj.__set_lang__(lang,REQUEST['lang_label'],'',manage_lang)
        
    ### Init Meta-Dictionaries.
    xml = open(package_home(globals())+'/import/metadata.default.conf.'+manage_lang+'.xml')
    _metadictmanager.importXml(obj,xml=xml)
    xml.close()

    ### Init Text-Formats
    xml = open(package_home(globals())+'/import/textFmt.xml')
    _textformatmanager.importXml(obj,xml=xml)
    xml.close()

    ### Init Object-Attributes.
    obj.synchronizeObjAttrs()
        
    ### Init Role-Definitions and Permission Settings.
    obj.initRoleDefs()

    ##### Object State ####
    obj.setObjStateNew(REQUEST)
    ##### Init Version (has to be called after setObjectStateNew!) ####
    obj.updateVersion(lang,manage_lang,REQUEST)
    ##### Init Properties ####
    obj.setObjProperty('active',1,lang)
    obj.setObjProperty('titleshort','ZMS home',lang)
    obj.setObjProperty('title','ZMS - ZOPE-based contentmanagement system for science, technology and medicine',lang)

    ##### VersionManager ####
    obj.onChangeObj(REQUEST,forced=1)
        
    ##### StandardObjects ####
    obj.setConfProperty('ZMSSqlDb.enabled',0)
    obj.setConfProperty('ZMSSysFolder.enabled',0)
    
    ##### [MODULES] Add Default Content ####
    if REQUEST.get('content',0) == 1:
      _globals.writeLog("[manage_addZMS]: Add Default Content...")
      # Import content.
      xmlfile = open(package_home(globals())+'/import/content.default.zip','rb')
      _importable.importFile(obj,xmlfile,REQUEST)
      xmlfile.close()

    ##### [MODULES] Add Example Database ####
    if REQUEST.get('exampledb',0) == 1:
      _globals.writeLog("[manage_addZMS]: Add Example Database...")
      # Import special-objects.
      xml = open(package_home(globals())+'/import/specObj_exampleDb.xml')
      _metaobjmanager.importXml(obj,xml=xml)
      xml.close()
      # Import content.
      xmlfile = open(package_home(globals())+'/import/specObj_exampleDb.scott.xml','rb')
      _importable.importFile(obj,xmlfile,REQUEST)
      xmlfile.close()

    ##### [MODULES] Add Forum / Bulletin Board ####
    if REQUEST.get('specObj_discussions',0) == 1:
      if _globals.debug: print "[manage_addZMS]: Add Forum / Bulletin Board..."
      # Import special-objects.
      xml = open(package_home(globals())+'/import/specObj_discussions.xml')
      _metaobjmanager.importXml(obj,xml=xml)
      xml.close()
      # Import module.
      id = 'forum'
      obj.initMetapattern(id)
      # Instantiate module.
      _sort_id = 9900
      custom = obj.getMetapattern(id,REQUEST)['title']
      obj.manage_instantiateMetapattern(lang,manage_lang,_sort_id,custom,REQUEST)

    ##### [MODULES] Add Newsletter ####
    if REQUEST.get('specObj_newsletter',0) == 1:
      if _globals.debug: print "[manage_addZMS]: Add Newsletter..."
      # Import special-objects.
      xml = open(package_home(globals())+'/import/specObj_newsletter.xml')
      _metaobjmanager.importXml(obj,xml=xml)
      xml.close()
      obj.setConfProperty('newsletter_item.enabled',0)
      # Import actions.
      xml = open(package_home(globals())+'/import/manage_sendNewsletterItem.xml')
      _metacmdmanager.importXml(obj,xml=xml)
      xml.close()
      # Import module.
      id = 'newsletter'
      obj.initMetapattern(id)
      # Instantiate module.
      _sort_id = 9900
      custom = obj.getMetapattern(id,REQUEST)['title']
      obj.manage_instantiateMetapattern(lang,manage_lang,_sort_id,custom,REQUEST)
      # Remove module.
      obj.delMetapattern(id)      

    ##### [MODULES] Add Special Object Calendar ####
    if REQUEST.get('specObj_calendar',0) == 1:
      if _globals.debug: print "[manage_addZMS]: Add Special Object Calendar..."
      # Import special-objects.
      xml = open(package_home(globals())+'/import/specObj_calendar.xml')
      _metaobjmanager.importXml(obj,xml=xml)
      xml.close()

    # Return with message.
    message = obj.getLangStr('MSG_INSERTED',manage_lang)%obj.meta_type
    RESPONSE.redirect('%s/%s/manage?manage_tabs_message=%s'%(homeElmnt.absolute_url(),obj.id,urllib.quote(message)))
  
  else:
    RESPONSE.redirect('%s/manage_main'%self.absolute_url())


###################################################################################################
###################################################################################################
###   
###   C l a s s 
###   
###################################################################################################
###################################################################################################
class ZMS(
	zmscontainerobject.ZMSContainerObject,
	_printablemanager.PrintableManager,
	_builder.Builder,
	_cachemanager.CacheManager,
	_ftpmanager.FtpManager,
	_multilangmanager.MultiLanguageManager,
	_metadictmanager.MetadictManager,
	_metacmdmanager.MetacmdManager,
	_metaobjmanager.MetaobjManager,
	_metapatternmanager.MetapatternManager,
	_objattrs.ObjAttrsManager,
	_textformatmanager.TextFormatManager,
	_xsllib.XslManager,
	_zcatalogmanager.ZCatalogManager,
	):

    # Version-Info.
    # ------------
    zms_version = '1.3'
    zms_build = '116'
    zms_patch = 'j'

    # Properties.
    # -----------
    meta_type = 'ZMS'
    icon = "misc_/zms/zms.gif"
    icon_disabled = 'misc_/zms/zms_disabled.gif'

    # Management Options.
    # -------------------
    manage_options = (
	{'label': 'TAB_EDIT',         'action': 'manage_main'},
	{'label': 'TAB_PROPERTIES',   'action': 'manage_properties'},
	{'label': 'TAB_ACCESS',       'action': 'manage_users'},
	{'label': 'TAB_IMPORTEXPORT', 'action': 'manage_importexport'},
	{'label': 'TAB_TASKS',        'action': 'manage_tasks'},
	{'label': 'TAB_REFERENCES',   'action': 'manage_RefForm'},
	{'label': 'TAB_HISTORY',      'action': 'manage_UndoVersionForm'},
	{'label': 'TAB_CONFIGURATION','action': 'manage_customizeForm'},
	{'label': 'TAB_PREVIEW',      'action': 'preview_html'}, # empty string defaults to index_html
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main','manage_workspace',
		'manage_addZMSRubrik','manage_addZMSDocument','manage_addZMSTextarea','manage_addZMSGraphic','manage_addZMSTable','manage_addZMSFile','manage_addZMSTeaserContainer','manage_addZMSNote','manage_addZMSLinkContainer','manage_addZMSLinkElement','manage_addZMSSqlDb',
		'manage_deleteObjs','manage_moveObjUp','manage_moveObjDown','manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_properties','manage_changeProperties',
		'manage_search','manage_tasks',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		'manage_users',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		'manage_importexport','manage_import','manage_export',
		)
    __administratorPermissions__ = (
		'manage_addZMSSysFolder',
		'manage_customizeForm','manage_customizeSystem','manage_updateCatalog',
		'manage_changeLanguages','manage_customizeLanguagesForm',
		'manage_changeMetapattern','manage_customizeMetapatternForm',
		'manage_changeMetaobjs','manage_customizeMetaobjForm',
                'manage_changeMetadicts','manage_customizeMetadictForm',
		'manage_changeMetacmds','manage_customizeMetacmdForm',
                'manage_customizeTextFormatForm','manage_customizeTextFormat',
		'manage_customizeDesignForm','manage_customizeDesign',
		)
    __ac_permissions__=(
		('ZMS Administrator', __administratorPermissions__),
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'string','multilang':1,'lang_inherit':_globals.false},
        'change_dt':{'datatype':'datetime','multilang':1,'lang_inherit':_globals.false},
        # Versioned by
        'work_uid':{'datatype':'string','multilang':1,'lang_inherit':_globals.false},
        'work_dt':{'datatype':'datetime','multilang':1,'lang_inherit':_globals.false},
        # Properties
        'active':{'datatype':'boolean','multilang':1},
        'attr_active_start':{'datatype':'datetime','multilang':1},
        'attr_active_end':{'datatype':'datetime','multilang':1},
        'title':{'datatype':'string','multilang':1,'size':40},
        'titleshort':{'datatype':'string','multilang':1,'size':20},
        'titleimage':{'datatype':'image','multilang':1},
        'levelnfc':{'datatype':'string','type':'select','options':[0,0,1,1,2,2],'label':'ATTR_LEVELNFC'},
        'attr_cacheable':{'datatype':'boolean','default':1},
        # Meta-Dictionaries        
        '$metadict':{'datatype':'MetaDict'},
    }

    # Globals.
    # --------
    dGlobalAttrs = {
	'ZMS':{'obj_class':None,'meta_dict':1,'icon':icon,'constructor':None,'obj_attrs':__obj_attrs__},
	'ZMSCustom':{'obj_class':ZMSCustom,'meta_dict':0,'icon':ZMSCustom.icon,'constructor':None,'obj_attrs':ZMSCustom.__obj_attrs__},
	'ZMSDocument':{'obj_class':ZMSDocument,'meta_dict':1,'icon':ZMSDocument.icon,'constructor':'manage_addzmsdocumentform','obj_attrs':ZMSDocument.__obj_attrs__},
	'ZMSFile':{'obj_class':ZMSFile,'meta_dict':1,'icon':ZMSFile.icon,'constructor':'manage_addzmsfileform','obj_attrs':ZMSFile.__obj_attrs__},
	'ZMSGraphic':{'obj_class':ZMSGraphic,'meta_dict':1,'icon':ZMSGraphic.icon,'constructor':'manage_addZMSGraphic','obj_attrs':ZMSGraphic.__obj_attrs__},
	'ZMSLinkContainer':{'obj_class':ZMSLinkContainer,'meta_dict':0,'icon':ZMSLinkContainer.icon,'constructor':'manage_addZMSLinkContainer','obj_attrs':ZMSLinkContainer.__obj_attrs__},
	'ZMSLinkElement':{'obj_class':ZMSLinkElement,'meta_dict':1,'icon':ZMSLinkElement.icon,'constructor':'manage_addzmslinkelementform','obj_attrs':ZMSLinkElement.__obj_attrs__},
	'ZMSNote':{'obj_class':ZMSNote,'meta_dict':0,'icon':ZMSNote.icon,'constructor':'manage_addZMSNote','obj_attrs':ZMSNote.__obj_attrs__},
	'ZMSRubrik':{'obj_class':ZMSRubrik,'meta_dict':1,'icon':ZMSRubrik.icon,'constructor':'manage_addzmsrubrikform','obj_attrs':ZMSRubrik.__obj_attrs__},
	'ZMSSqlDb':{'obj_class':ZMSSqlDb,'meta_dict':0,'icon':ZMSNote.icon,'constructor':'manage_addZMSSqlDb','obj_attrs':ZMSSqlDb.__obj_attrs__},
	'ZMSSysFolder':{'obj_class':ZMSSysFolder,'meta_dict':1,'icon':ZMSSysFolder.icon,'constructor':'manage_addzmssysfolderform','obj_attrs':ZMSSysFolder.__obj_attrs__},
	'ZMSTable':{'obj_class':ZMSTable,'meta_dict':1,'icon':ZMSTable.icon,'constructor':'manage_addzmstableform','obj_attrs':ZMSTable.__obj_attrs__},
	'ZMSTeaserElement':{'obj_class':ZMSTeaserElement,'meta_dict':0,'icon':ZMSTeaserElement.icon,'constructor':'manage_addZMSTeaserElement','obj_attrs':ZMSTeaserElement.__obj_attrs__},
	'ZMSTeaserContainer':{'obj_class':ZMSTeaserContainer,'meta_dict':0,'icon':ZMSTeaserContainer.icon,'constructor':'manage_addZMSTeaserContainer','obj_attrs':ZMSTeaserContainer.__obj_attrs__},
	'ZMSTextarea':{'obj_class':ZMSTextarea,'meta_dict':1,'icon':ZMSTextarea.icon,'constructor':'manage_addZMSTextarea','obj_attrs':ZMSTextarea.__obj_attrs__},
	'ZMSTrashcan':{'obj_class':ZMSTrashcan,'meta_dict':0,'icon':ZMSTrashcan.icon,'constructor':None,'obj_attrs':ZMSTrashcan.__obj_attrs__},
	}

    # Register DTML-Templates.
    # ------------------------
    index_html = HTMLFile('dtml/zms/index', globals()) # index_html
    f_inactive_html = HTMLFile('dtml/zms/f_inactive', globals()) # inactive_html
    f_headDoctype = HTMLFile('dtml/zms/f_headdoctype', globals()) # Template_L1: DOCTYPE
    f_bodyContent = HTMLFile('dtml/zms/f_bodycontent', globals()) # Template: Body-Content / Element
    f_bodyContent_Float = HTMLFile('dtml/zms/f_bodycontent_float', globals()) # Template: Body-Content / Textkrper
    f_bodyContent_Sitemap = HTMLFile('dtml/zms/f_bodycontent_sitemap', globals()) # Template: Sitemap
    f_bodyContent_Search = HTMLFile('dtml/zms/f_bodycontent_search', globals()) # Template: Search
    f_headTitle = HTMLFile('dtml/zms/f_headtitle', globals()) # Head.Title
    f_headMeta_DC = HTMLFile('dtml/zms/f_headmeta_dc', globals()) # Head.Meta.DC
    headScript = HTMLFile('dtml/zms/headscript', globals()) # Head.Script
    headMeta = HTMLFile('dtml/zms/headmeta', globals()) # Head.Meta
    headCStyleSheet = HTMLFile('dtml/zms/headcstylesheet', globals()) # Template_L1: CSS-Referenz
    headCSS = HTMLFile('dtml/zms/headcstylesheet', globals()) # Template_L1: CSS-Referenz
    f_sitemap = HTMLFile('dtml/zms/f_sitemap', globals()) # f_sitemap
    f_standard_html_request = HTMLFile('dtml/zms/f_standard_html_request', globals()) # f_standard_html_request
    f_standard_html_header = HTMLFile('dtml/zms/f_standard_html_header', globals()) # f_standard_html_header
    f_standard_html_footer = HTMLFile('dtml/zms/f_standard_html_footer', globals()) # f_standard_html_footer
    search_nav_html = HTMLFile('dtml/zms/search_nav', globals()) # search_nav_html


    """
    ###############################################################################################    
    #
    #   DECLARATIVE URLS
    #
    #   Based on an idea of D. Datzert (dirk.datzert@tks-rasselstein.thyssenkrupp.com]) and the 
    #   Zope-Product PathHandler (http://www.zope.org/Members/NIP/PathHandler).
    #
    ###############################################################################################    
    """

    # ------------------------------------------------------------------------------------------
    #  ZMS.__bobo_traverse__
    # ------------------------------------------------------------------------------------------
    def __bobo_traverse__(self, TraversalRequest, name):
      # If this is the first time this __bob_traverse__ method has been called
      # in handling this traversal request, store the path_to_handle

      if not TraversalRequest.has_key('path_to_handle'):
          # Make a reversed copy of the TraversalRequestNameStack
          TraversalRequestNameStackReversed=copy.copy(TraversalRequest['TraversalRequestNameStack'])
          TraversalRequestNameStackReversed.reverse()

          # Set path_to_handle in the TraversalRequest.
          TraversalRequest['path_to_handle']=[name]+TraversalRequestNameStackReversed

      # If the name is in the list to exclude, call it.
      path_ob = None
      if name=='manage':
        path_to_handle = TraversalRequest.get('path_to_handle',[])
        if len(path_to_handle) > 1 and path_to_handle[-1]=='manage':
          path_ob = self.pathob(path_to_handle[:-1],TraversalRequest)
      if path_ob is not None:
        return self.pathhandler
        
      # If the name is in the list of attributes, call it.
      elif getattr(self,name,None) is not None:
        return getattr(self,name)

      # otherwise do some 'magic'
      else:
        # If there's no more names left to handle, return the path handling method
        # to the traversal machinery so it gets called next
        if len(TraversalRequest['TraversalRequestNameStack'])==0:
          return self.pathhandler
        # If there are more names to handle, return ourself, so this __bobo_traverse__ gets
        # called again and we get to ignore the other names.
        else:
          return self


    # ------------------------------------------------------------------------------------------
    #  ZMS.__call__
    # ------------------------------------------------------------------------------------------
    def __call__(self, REQUEST={}, **kw):
      """"""
      # This handles the special case where the Path Handler's id is the last one in the
      # traversal request.
      # eg: when the URL ends with the id of the Path Handler.
      # > REQUEST['path_to_handle']=[]
      # > return self.pathhandler(REQUEST)
      return self      


    # ------------------------------------------------------------------------------------------
    #  ZMS.pathob
    # ------------------------------------------------------------------------------------------
    def pathob(self, path_to_handle, REQUEST):
      if self.getConfProperty('ZMS.pathhandler',0)==1:
        langs = self.getLanguages()
        path_ob = self.getDocumentElement()
        path_index = 0
        while 1:
          if path_index == len(path_to_handle):
            return path_ob
          path_item = path_to_handle[path_index].lower()
          if path_index+1 == len(path_to_handle) and path_item in langs:
            lang = path_item
            REQUEST.set('lang',lang) 
            return path_ob
          this_ob = path_ob
          path_ob = None
          for lang in langs:
            if path_ob is None:
              req = {'lang':lang,'preview':REQUEST.get('preview','')}
              obs = this_ob.getObjChildren('',req)
              for ob in obs:
                id = ob.id
                titlealt = ob.getObjProperty('attr_dc_identifier_url_node',req)
                if len(titlealt)==0:
                  titlealt = ob.getTitlealt(req)
                if path_item == id or path_item == self.id_quote(titlealt):
                  path_ob = ob
              if path_ob is not None:
                REQUEST.set('lang',lang)
          if path_ob is None: 
            break
          path_index = path_index + 1 
      return None


    # ------------------------------------------------------------------------------------------
    #  ZMS.pathhandler
    # ------------------------------------------------------------------------------------------
    def pathhandler(self, REQUEST, RESPONSE=None):
      """ ZMS.pathhandler """
      path_to_handle = REQUEST.get('path_to_handle',[])
      if len(path_to_handle) > 0 and path_to_handle[-1]=='manage':
        path_ob = self.pathob(path_to_handle[:-1],REQUEST)
        return REQUEST.RESPONSE.redirect(path_ob.absolute_url()+'/'+path_to_handle[-1])
      else:
        path_ob = self.pathob(path_to_handle,REQUEST)
        if path_ob is not None:
          path_ob = zmscontainerobject.getPageWithElements(path_ob,REQUEST)
          return self.index_html(path_ob,REQUEST)
        else:
          raise "Resource not found",'<h2>Site-Error</h2><b>Sorry, there is no web page matching your request.</b> It is possible you typed the address incorrectly, or that the page no longer exists.<hr><b>Resource<b> '+''.join(map(lambda x: x+'/',path_to_handle))+' GET'


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """
    
    # ------------------------------------------------------------------------------------------
    #  ZMS.__init__: 
    #
    #  Constructor (initialise a new instance of ZMS).
    # ------------------------------------------------------------------------------------------
    def __init__(self, id): 
      """ ZMS.__init__ """
      self.id = id
      self.logo = ImageFile(_fileutil.getOSPath(package_home(globals())+'/images/logo.gif'))


    # ------------------------------------------------------------------------------------------
    #  ZMS.recurse_updateVersion:
    #
    #  Update version.
    # ------------------------------------------------------------------------------------------
    def recurse_updateVersion(self, REQUEST):
      message = ''
      
      build = getattr(self,'build','000')      

      # ---------------------------------------------------------------------
      # 13.12.2002
      # Build #106: Configuration.
      # ---------------------------------------------------------------------
      #-- ZMS.autocommit
      if hasattr(self,'autocommit'):
        self.setConfProperty('ZMS.autocommit',getattr(self,'autocommit',1))
        try: delattr(self,'autocommit')
        except: pass
      #-- ZMS.custom.*
      if hasattr(self,'__attr_dc_metadicts__'):
        self.setConfProperty('ZMS.custom.metas',copy.deepcopy(getattr(self,'__attr_dc_metadicts__',[])))
        try: delattr(self,'__attr_dc_metadicts__')
        except: pass
      if hasattr(self,'__attr_dc_metaobjs__'):
        self.setConfProperty('ZMS.custom.objects',getattr(self,'__attr_dc_metaobjs__',{}).copy())
        try: delattr(self,'__attr_dc_metaobjs__')
        except: pass
      if hasattr(self,'__attr_lang_dict__'):
        self.setConfProperty('ZMS.custom.langs.dict',getattr(self,'__attr_lang_dict__',{}).copy())
        try: delattr(self,'__attr_lang_dict__')
        except: pass
      #-- ZMS.cache.*
      if hasattr(self,'__cache_active__'):
        self.setConfProperty('ZMS.cache.active',getattr(self,'__cache_active__',0))
        try: delattr(self,'__cache_active__')
        except: pass
      #-- ZMS.catalog.*
      if hasattr(self,'__catalog_period__'):
        self.setConfProperty('ZMS.catalog.period',getattr(self,'__catalog_period__','0'))
        try: delattr(self,'__catalog_period__')
        except: pass
      #-- Zope.db.autopack.*
      if hasattr(self,'property_threshold'):
        self.setConfProperty('Zope.db.autopack.threshold',getattr(self,'property_threshold',None))
        try: delattr(self,'property_threshold')
        except: pass
      if hasattr(self,'property_days'):
        self.setConfProperty('Zope.db.autopack.days',getattr(self,'property_days',None))
        try: delattr(self,'property_days')
        except: pass
      #-- ASP.*
      if hasattr(self,'asp_ip_or_domain'):
        self.setConfProperty('ASP.ip_or_domain',self.getConfProperty('ASP.ip_or_domain',getattr(self,'asp_ip_or_domain','')))
        try: delattr(self,'asp_ip_or_domain')
        except: pass
      if hasattr(self,'asp_uid'):
        self.setConfProperty('ASP.login',self.getConfProperty('ASP.login',getattr(self,'asp_uid','')))
        try: delattr(self,'asp_uid')
        except: pass
      if hasattr(self,'asp_pwd'):
        self.setConfProperty('ASP.password',self.getConfProperty('ASP.password',getattr(self,'asp_pwd','')))
        try: delattr(self,'asp_pwd')
        except: pass

      # -----------------------------------------------------------
      # 09.03.2002
      # Build #114: Deactivate ZMSSqlDb.
      # -----------------------------------------------------------
      if getattr(self.getDocumentElement(),'build','') < '114':
        self.setConfProperty('ZMSSqlDb.enabled',0)

      # -----------------------------------------------------------
      # 19.03.2002
      # Build #116: Textformats.
      # -----------------------------------------------------------
      if hasattr(self,'__attr_text_formats__'):
        self.setConfProperty('ZMS.custom.textformats',copy.deepcopy(getattr(self,'__attr_text_formats__')))
        delattr(self,'__attr_text_formats__')
      if hasattr(self,'__attr_text_formats_default__'):
        self.setConfProperty('ZMS.custom.textformats.default',getattr(self,'__attr_text_formats_default__','body'))
        delattr(self,'__attr_text_formats_default__')

      # Process clients.
      # ----------------
      thisHome = self.getHome()
      for id in self.getConfProperty('Portal.Clients',[]):
        try:
          clientHome = getattr(thisHome,id,None)
          clientDocElmnt = clientHome.objectValues(['ZMS'])[0]
          message = message + "<i>%s</i>: %s<br>"%(id,clientDocElmnt.recurse_updateVersion(REQUEST))
        except:
          message = message + "<i>%s</i>: Exception!<br>"%(id)

      # Process object.
      # ---------------
      self.getStylesheet()
      self.getSequence()
      self.recurse_updateVersionMetaDicts(REQUEST)
      self.recurse_updateVersionMetaObjs()
      self.recurse_updateVersionLanguages(REQUEST)
      self.synchronizeObjAttrs()
      self.initRoleDefs()

      # Process import-artefacts.
      # -------------------------
      message = message + _importable.recurse_afterImport(self)
      
      # Process super-objects.
      # ----------------------
      message = message + zmscontainerobject.ZMSContainerObject.recurse_updateVersion(self,REQUEST)

      # Update build-id.
      # ----------------
      setattr(self,'build',self.zms_build)

      # Return with message.            
      message = message + 'Updated object-attributes from Build #%s to newer Build #%s!'%(build,self.zms_build)
      return message


    # ---------------------------------------------------------------------------------------------
    #  ZMS.getDocumentElement:
    #
    #  The root element of the site.
    # ---------------------------------------------------------------------------------------------
    def getDocumentElement(self):
      return self

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getHome:
    #
    #  Returns the home-folder of the site.
    # ---------------------------------------------------------------------------------------------
    def getHome(self):
      return self.aq_parent
    
    # ---------------------------------------------------------------------------------------------
    #  ZMS.getSequence:
    #
    #  Returns sequence (Sequence).
    # ---------------------------------------------------------------------------------------------
    def getSequence(self):
      id = 'acl_sequence'
      if not id in self.objectIds(['Sequence']):
        _sequence.manage_addSequence(self,startvalue=_getNewId(self,0))
      return getattr(self,id)

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getStylesheet:
    #
    #  Returns stylesheet (DTML Method).
    # ---------------------------------------------------------------------------------------------
    def getStylesheet(self, defaultCSS=''):
      id = 'stylesheet.css'
      if not id in self.objectIds(['DTML Method']):
        title = '*** DO NOT DELETE OR MODIFY ***'
        if len(defaultCSS) == 0:
          defaultCSS = HTMLFile('dtml/zms/__init__stylesheet', globals())(self)
        raw = getattr(self,'stylesheet',defaultCSS)
        self.manage_addDTMLMethod(id,title,raw)
        try: delattr(self,'stylesheet')
        except: pass
      return getattr(self,id)

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getNewId:
    #
    #  Returns new (unique) Object-ID.
    # ---------------------------------------------------------------------------------------------
    def getNewId(self, id_prefix='e'): 
      return '%s%i'%(id_prefix,self.getSequence().nextVal())

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getMetaTypes:
    #
    #  Returns list of meta-types.
    # ---------------------------------------------------------------------------------------------
    def getMetaTypes(self, REQUEST, excl_meta_types=['ZMS','ZMSCustom','ZMSTrashcan']): 
      meta_types = []
      for meta_type in self.dGlobalAttrs.keys():
        if meta_type not in excl_meta_types:
          meta_types.append((self.display_type(REQUEST,meta_type),meta_type))
      meta_types.extend(map(lambda meta_type: (self.display_type(REQUEST,meta_type),meta_type),self.getMetaobjIds()))
      meta_types.sort()
      return meta_types

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getDCCoverage
    # ---------------------------------------------------------------------------------------------
    def getDCCoverage(self, REQUEST={}): 
      return 'global.'+self.getPrimaryLanguage()

    # ---------------------------------------------------------------------------------------------
    #  ZMS.sendMail:
    #
    #  Sends Mail via MailHost.
    # ---------------------------------------------------------------------------------------------
    def sendMail(self, mto, msubject, mbody, REQUEST):
    
      ##### Get Sender ####
      auth_user = REQUEST['AUTHENTICATED_USER']
      mfrom = str(getattr(auth_user,'email',self.getConfProperty('ZMSAdministrator.email','')))
    
      ##### Get MailHost ####
      mailhost = None
      homeElmnt = self.getHome()
      if len(homeElmnt.objectValues(['Mail Host'])) == 1:
        mailhost = homeElmnt.objectValues(['Mail Host'])[0]
      elif getattr(homeElmnt,'MailHost',None) is not None:
        mailhost = getattr(homeElmnt,'MailHost',None)

      ##### Get MessageText ####
      messageText = ''
      messageText = messageText + 'From: %s\n'%mfrom
      messageText = messageText + 'To: %s\n'%mto
      messageText = messageText + 'Subject: %s\n\n'%msubject
      messageText = messageText + '%s\n'%mbody
      if _globals.debug: print "[sendMail]: %s"%messageText

      ##### Send mail ####
      if mailhost is not None and len(mto) > 0 and len(mfrom) > 0:
        try:
          mailhost.send(messageText)
        except:
          pass



    """
    ###############################################################################################    
    #
    #   ZMS - System Configuration
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #  ZMS.__get_attr_conf_dict__:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def __get_attr_conf_dict__(self):
      return getattr(self,'__attr_conf_dict__',{})

    ###############################################################################################
    #  ZMS.manage_exportConfProperties:
    #
    #  Returns XML-export of properties from configuration.
    ###############################################################################################
    def manage_exportConfProperties(self, REQUEST, RESPONSE):
      """ manage_exportConfProperties """
      attr_conf_dict = self.__get_attr_conf_dict__()
      filename = 'CONF.xml'
      content_type = 'text/xml'
      export = '<?xml version="1.0" encoding="ISO-8859-1"?>'
      export = export + _xmllib.toXml(self,attr_conf_dict)
      RESPONSE.setHeader('Content-Type',content_type)
      RESPONSE.setHeader('Content-Disposition','inline;filename=%s'%filename)
      return export

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getConfProperty:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def getConfProperty(self, key, default=None):
      v = default
      attr_conf_dict = self.__get_attr_conf_dict__()
      if attr_conf_dict.has_key(key):
        v = attr_conf_dict[key]
      return v

    # ---------------------------------------------------------------------------------------------
    #  ZMS.setConfProperty:
    #
    #  Returns property from configuration.
    # ---------------------------------------------------------------------------------------------
    def setConfProperty(self, key, value):
      attr_conf_dict = self.__get_attr_conf_dict__()
      attr_conf_dict[key] = value
      self.__attr_conf_dict__ = attr_conf_dict
      self.__attr_conf_dict__ = self.__attr_conf_dict__.copy()

    # ---------------------------------------------------------------------------------------------
    #  ZMS.getMediaDb:
    #
    #  Returns mediadb (MediaDb).
    # ---------------------------------------------------------------------------------------------
    def getMediaDb(self):
      id = 'acl_mediadb'
      return getattr(self,id,None)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getAutocommit
    # ---------------------------------------------------------------------------------------------
    def getAutocommit(self): 
      return self.getConfProperty('ZMS.autocommit',1)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.doAutocommit:
    #
    #	Auto-Commit ZMS-tree.
    # ---------------------------------------------------------------------------------------------
    def doAutocommit(self, lang, manage_lang, REQUEST):
      _versionmanager.doAutocommit(self,REQUEST)

    # ---------------------------------------------------------------------------------------------
    #	ZMS.doPack:
    #
    #	Pack ZMS-database.
    # ---------------------------------------------------------------------------------------------
    def doPack(self, lang, manage_lang, REQUEST, forced=0):
      message = ''
      
      # Clear objects from trashcan.
      # ----------------------------
      try:
        period = self.getConfProperty('ZMS.Version.autopack','')
        dt = self.getConfProperty('ZMS.Version.autopack.datetime',None)
        trashcan = self.getTrashcan()
        ids = []
        for ob in trashcan.objectValues(self.dGlobalAttrs.keys()):
          delete = 1
          for s_lang in self.getLanguages():
            req = {'lang':s_lang,'preview':'preview'}
            change_dt = ob.getObjProperty('change_dt',req)
            if dt is not None and change_dt is not None:
              delete = delete and _globals.daysBetween(change_dt,dt)>=int(period)
          if delete:
            ids.append(ob.id)
        l=len(ids)
        trashcan.manage_delObjects(ids=ids)
        message = message + 'Cleared %i objects from trashcan.<br>'%l
      except:
        message = message + 'Exception when clearing objects from trashcan.<br>'

      # Remove unused versions from history.
      # ------------------------------------
      try:
        _versionmanager.doPack(self,REQUEST)
        message = message + 'Remove unused versions from history.<br>'
      except:
        message = message + 'Exception when removing unused versions from history.<br>'
        
      # Update time-stamp.
      self.setConfProperty('ZMS.Version.autopack.datetime',DateTime())

      # Return with message.
      return message


    # ---------------------------------------------------------------------------------------------
    #	ZMS.doAutopack:
    #
    #	Auto-Pack ZODB.
    # ---------------------------------------------------------------------------------------------
    def doAutopack(self, lang, manage_lang, REQUEST):
      
      # ZMS.
      # ----
      active = self.getConfProperty('ZMS.Version.active',0)
      period = self.getConfProperty('ZMS.Version.autopack','')
      dt = self.getConfProperty('ZMS.Version.autopack.datetime',None)
      if active==1 and len(period)>0 and (dt is None or not dt.isCurrentDay()):
        self.doPack(lang,manage_lang,REQUEST)
      
      # ZODB.
      # -----
      active = self.getConfProperty('ZODB.autopack.active',0)
      dt = self.getConfProperty('ZODB.autopack.datetime',None)
      if active==1 and (dt is None or not dt.isCurrentDay()):
        try:
          self.Control_Panel.Database.manage_pack(0)
        except:
          pass
        self.setConfProperty('ZODB.autopack.datetime',DateTime())


    ###############################################################################################
    # ZMS.manage_recurseAfterImport: 
    ###############################################################################################
    def manage_recurseAfterImport(self, REQUEST=None, RESPONSE=None): 
      """ ZMS.manage_recurseAfterImport """
      _importable.recurse_afterImport(self)

    
    ###############################################################################################
    # ZMS.manage_customizeSystem: 
    #
    # Change system properties.
    ###############################################################################################
    def manage_customizeSystem(self, btn, key, lang, manage_lang, REQUEST, RESPONSE): 
      """ ZMS.manage_customizeSystem """

      message = ''
      
      ##### ASP ####
      if key == 'ASP':
        self.setConfProperty('ZMSAdministrator.email',REQUEST.get('zmsadministrator_email',''))
        self.setConfProperty('ASP.ip_or_domain',REQUEST.get('asp_ip_or_domain',''))
        self.setConfProperty('ASP.login',REQUEST.get('asp_login',''))
        if len(REQUEST.get('asp_password','')) > 0 and REQUEST.get('asp_password','') == REQUEST.get('asp__confirm',''):
          self.setConfProperty('ASP.password',REQUEST.get('asp_password',''))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### History ####
      elif key == 'History':
        self.setConfProperty('ZMS.Version.active',REQUEST.get('active',0))
        self.setConfProperty('ZMS.Version.autopack',REQUEST.get('autopack',''))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
        if self.getConfProperty('ZMS.Version.active',0)==0 or btn == 'Pack':
          message = self.doPack(lang,manage_lang,REQUEST)

      ##### Workflow ####
      elif key == 'Workflow':
        old_autocommit = self.getAutocommit()
        new_autocommit = REQUEST.get('workflow',0)==0
        self.setConfProperty('ZMS.autocommit',new_autocommit)
        if old_autocommit == 0 and new_autocommit == 1:
          self.doAutocommit(lang,manage_lang,REQUEST)
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### Version ####
      elif key == 'Version':
        if btn == 'Regenerate':
          message = self.recurse_updateVersion(REQUEST)
      
      ##### Clients ####
      elif key == 'Clients':
        if btn == 'Change':
          s = REQUEST.get('portal_master','').strip()
          if s != self.getHome().id:
            self.setConfProperty('Portal.Master',s)
          l = []
          for s in REQUEST.get('portal_clients','').split('\n'):
            s = s.strip()
            if s in self.getHome().objectIds(['Folder']):
              l.append(s)
          self.setConfProperty('Portal.Clients',l)
          message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### AutoPack ####
      elif key == 'AutoPack':
        if btn == 'Change':
          self.setConfProperty('ZODB.autopack.active',REQUEST.get('active',0))
          message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      ##### MediaDb ####
      elif key == 'MediaDb':
        if btn == 'Create':
          location = REQUEST['mediadb_location'].strip()
          _mediadb.manage_addMediaDb(self,location)
        elif btn == 'Remove':
          _mediadb.manage_delMediaDb(self)
        message = self.getLangStr('MSG_CHANGED',manage_lang)
          
      ##### Custom ####
      elif key == 'Custom':
        if btn == 'Change':
          self.setConfProperty(REQUEST['option'],REQUEST.get('value',''))

      ##### InstalledProducts ####
      elif key == 'InstalledProducts':
        if btn == 'Change':
          self.setConfProperty('ZMS.demo',REQUEST.get('demo_active',0))
          for product in ['jdk12','pil','htmldoc']:
            self.setConfProperty('%s.%s'%(key,product),REQUEST.get(product,None))
          self.setConfProperty('InstalledProducts.pil.scale',REQUEST.get('pil_scale',0))
          self.setConfProperty('InstalledProducts.pil.thumbnail',REQUEST.get('pil_thumbnail',0))
          self.setConfProperty('InstalledProducts.pil.thumbnail.max',REQUEST.get('pil_thumbnail_max',100))
          self.setConfProperty('InstalledProducts.pil.superres',REQUEST.get('pil_superres',0))
          self.setConfProperty('InstalledProducts.pil.hires.thumbnail.max',REQUEST.get('pil_hires_thumbnail_max',600))
          message = self.getLangStr('MSG_CHANGED',manage_lang)
          
      ##### StandardObjects ####
      elif key == 'StandardObjects':
        meta_types = []
        meta_types.extend(self.dGlobalAttrs.keys())
        meta_types.extend(self.getMetaobjIds())
        for meta_type in meta_types:
          self.setConfProperty('%s.enabled'%meta_type,meta_type in REQUEST.get('meta_types',[]))
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeForm?lang=%s&manage_lang=%s&option=%s&manage_tabs_message=%s#_%s'%(lang,manage_lang,str(REQUEST.get('option','')),message,key))


    """
    ###############################################################################################    
    #
    #   ZMS - Versions
    #
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.updateVersion:
    #
    #	Update version.
    # ---------------------------------------------------------------------------------------------
    def updateVersion(self, lang, manage_lang, REQUEST):
      #++ if _globals.debug: print "[%s.updateVersion]"%(self.meta_type)
      message = ''
      build = getattr(self,'build','000')
      if build < self.zms_build:
        message = self.recurse_updateVersion(REQUEST)
      self.doAutopack(lang,manage_lang,REQUEST)
      return message


    """
    ###############################################################################################    
    #
    #   Customize
    #
    ###############################################################################################    
    """

    # Customize Interfaces.
    manage_customizeForm = HTMLFile('dtml/zms/manage_customizeform', globals()) 

    ###############################################################################################
    # ZMS.manage_customizeDesign: 
    #
    # Change layout (CSS Stylesheet and Logo).
    ###############################################################################################
    manage_customizeDesignForm = HTMLFile('dtml/zms/manage_customizedesignform', globals()) 
    def manage_customizeDesign(self, lang, manage_lang, REQUEST, RESPONSE):
      """ ZMS.manage_customizeDesign """
      
      if REQUEST['btn'] == self.getLangStr('BTN_CHANGE',manage_lang):
      
        # Change Stylesheet.
        # ------------------
        if REQUEST.has_key('stylesheet'):
          method = self.getStylesheet()
          data = REQUEST['stylesheet']
          title = method.title
          self.getStylesheet().manage_edit(data,title)
          
        # Change Layout of Sitemap.
        # -------------------------
        if REQUEST.has_key('attr_layoutsitemap'):
          if len(REQUEST['attr_layoutsitemap'])>0:
            self.attr_layoutsitemap = int(REQUEST['attr_layoutsitemap'])
          elif hasattr(self,'attr_layoutsitemap'):
            delattr(self,'attr_layoutsitemap')
      
      # Upload Logo.
      # ------------      
      elif REQUEST['btn'] == self.getLangStr('BTN_UPLOAD',manage_lang):
        self.logo = Image(id='logo',title='',file='')
        self.logo.manage_upload(REQUEST['file'],REQUEST)

      # Reset Logo.
      # -----------      
      elif REQUEST['btn'] == self.getLangStr('BTN_CLEAR',manage_lang):
        self.logo = ImageFile(_fileutil.getOSPath(package_home(globals())+'/images/logo.gif'))
      
      # Return with message.
      message = urllib.quote(self.getLangStr('MSG_CHANGED',manage_lang))
      return RESPONSE.redirect('manage_customizeDesignForm?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    """
    ###############################################################################################
    ###  
    ###  DOM-Methoden 
    ### 
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMS.getParentNode:
    #
    #	The parent of this node. 
    #	All nodes except root may have a parent.
    # ---------------------------------------------------------------------------------------------
    def getParentNode(self): 
      return None


    """
    ###############################################################################################    
    #
    #   XML-BUILDER
    #
    ###############################################################################################    
    """

    ###############################################################################################
    # ZMS.xmlOnStartElement(self, sTagName, dAttrs, oParentNode, oRoot):
    #
    # handler for XML-Builder (_builder.py)
    ###############################################################################################
    def xmlOnStartElement(self, sTagName, dTagAttrs, oParentNode, oRoot):
      """ZMS.xmlOnStartElement"""
      _globals.writeLog("[%s.xmlOnStartElement]: sTagName=%s"%(self.meta_type,sTagName))

      # remove all ZMS-objects.
      self.manage_delObjects(self.objectIds(self.dGlobalAttrs.keys()))
      # remove all languages.
      for s_lang in self.getLanguages():
        self.__del_lang__(s_lang)
      
      self.dTagStack = _globals.MyStack()
      self.dValueStack  = _globals.MyStack()
      
      # WORKAROUND! The member variable "aq_parent" does not contain the right parent object at 
      # this stage of the creation process (it will later on!). Therefore, we introduce a special
      # attribute containing the parent object, which will be used by xmlGetParent() (see below).
      self.oParent = None

###################################################################################################
#
# $Log: zms.py,v $
# Revision 1.1  2003/06/10 12:52:31  nordmann
# Initial import
#
#
